@@ -37,38 +37,13 @@ impl PublishCommand {
         let paths = crate::find_project_paths()?;
         let config = crate::config::root_config()?;
 
-        // Ask for confirmation if the package name if `gleam_*`
-        if config.name.starts_with("gleam_") && !config.name.starts_with("gleam_community_") {
-            println!(
-                "You are about to publish a package with a name that starts with
-the prefix `gleam_`, which is for packages maintained by the Gleam
-core team.",
-            );
-            let should_publish =
-                i_am_sure || cli::confirm("\nAre you sure you want to use this package name?")?;
-            if !should_publish {
-                println!("Not publishing.");
-                std::process::exit(0);
-            }
-            println!();
-        }
+        let should_publish = check_for_gleam_prefix(&config, i_am_sure)?
+            && check_for_version_zero(&config, i_am_sure)?
+            && check_repo_url(&config, i_am_sure)?;
 
-        // Ask for confirmation if the package is below version 1
-        if config.version.major == 0 {
-            println!(
-                "You are about to publish a release that is below version 1.0.0.
-
-Semantic versioning doesn't apply to version 0.x.x releases, so your 
-users will not be protected from breaking changes. This can result
-in a poor user experience where packages can break unexpectedly with 
-updates that would normally be safe."
-            );
-            let should_publish = i_am_sure || cli::confirm("\nDo you wish to continue?")?;
-            if !should_publish {
-                println!("Not publishing.");
-                std::process::exit(0);
-            }
-            println!();
+        if !should_publish {
+            println!("Not publishing.");
+            std::process::exit(0);
         }
 
         let Tarball {
@@ -78,6 +53,8 @@ updates that would normally be safe."
             generated_files_added,
         } = do_build_hex_tarball(&paths, &config)?;
 
+        check_for_name_squatting(&compile_result)?;
+
         // Build HTML documentation
         let docs_tarball =
             fs::create_tar_archive(docs::build_documentation(&config, &mut compile_result)?)?;
@@ -111,6 +88,95 @@ updates that would normally be safe."
     }
 }
 
+fn check_for_name_squatting(package: &Package) -> Result<(), Error> {
+    if package.modules.len() > 1 {
+        return Ok(());
+    }
+
+    let Some(module) = package.modules.first() else {
+        return Err(Error::HexPackageSquatting);
+    };
+
+    if module.dependencies.len() > 1 {
+        return Ok(());
+    }
+
+    let definitions = &module.ast.definitions;
+
+    if definitions.len() > 2 {
+        return Ok(());
+    }
+
+    let Some(main) = definitions.iter().find_map(|d| d.main_function()) else {
+        return Ok(());
+    };
+
+    if main.body.first().is_println() {
+        return Err(Error::HexPackageSquatting);
+    }
+
+    Ok(())
+}
+
+fn check_repo_url(config: &PackageConfig, i_am_sure: bool) -> Result<bool, Error> {
+    let Some(url) = config.repository.url() else {
+        return Ok(true);
+    };
+
+    let runtime = tokio::runtime::Runtime::new().expect("Unable to start Tokio async runtime");
+    let response = runtime.block_on(reqwest::get(&url)).map_err(Error::http)?;
+
+    if response.status().is_success() {
+        return Ok(true);
+    }
+
+    println!(
+        "The repository configuration in your `gleam.toml` file does not appear to be
+valid, {} returned status {}",
+        &url,
+        response.status()
+    );
+    let should_publish = i_am_sure || cli::confirm("\nDo you wish to continue?")?;
+    println!();
+    Ok(should_publish)
+}
+
+/// Ask for confirmation if the package name if a v0.x.x version
+fn check_for_version_zero(config: &PackageConfig, i_am_sure: bool) -> Result<bool, Error> {
+    if config.version.major != 0 {
+        return Ok(true);
+    }
+
+    println!(
+        "You are about to publish a release that is below version 1.0.0.
+
+Semantic versioning doesn't apply to version 0.x.x releases, so your
+users will not be protected from breaking changes. This can result
+in a poor user experience where packages can break unexpectedly with
+updates that would normally be safe."
+    );
+    let should_publish = i_am_sure || cli::confirm("\nDo you wish to continue?")?;
+    println!();
+    Ok(should_publish)
+}
+
+/// Ask for confirmation if the package name if `gleam_*`
+fn check_for_gleam_prefix(config: &PackageConfig, i_am_sure: bool) -> Result<bool, Error> {
+    if !config.name.starts_with("gleam_") || config.name.starts_with("gleam_community_") {
+        return Ok(true);
+    }
+
+    println!(
+        "You are about to publish a package with a name that starts with
+the prefix `gleam_`, which is for packages maintained by the Gleam
+core team.",
+    );
+    let should_publish =
+        i_am_sure || cli::confirm("\nAre you sure you want to use this package name?")?;
+    println!();
+    Ok(should_publish)
+}
+
 impl ApiKeyCommand for PublishCommand {
     fn with_api_key(
         &mut self,
@@ -209,6 +275,11 @@ fn do_build_hex_tarball(paths: &ProjectPaths, config: &PackageConfig) -> Result<
         return Err(Error::CannotPublishTodo { unfinished });
     }
 
+    // TODO: If any of the modules in the package contain a leaked internal type then
+    // refuse to publish as the package is not yet finished.
+    // We need to move aliases in to the type system first.
+    // context: https://discord.com/channels/768594524158427167/768594524158427170/1227250677734969386
+
     // Collect all the files we want to include in the tarball
     let generated_files = match target {
         Target::Erlang => generated_erlang_files(paths, &built.root_package)?,
