@@ -5,18 +5,21 @@ mod tests;
 use crate::{
     ast::{
         self, BitArrayOption, CustomType, Definition, DefinitionLocation, Function,
-        GroupedStatements, Import, ModuleConstant, RecordConstructor, RecordConstructorArg,
-        SrcSpan, TypeAlias, TypeAst, TypeAstConstructor, TypeAstFn, TypeAstHole, TypeAstTuple,
-        TypeAstVar, TypedDefinition, TypedModule, UntypedArg, UntypedModule, UntypedStatement,
+        GroupedStatements, Import, ModuleConstant, Publicity, RecordConstructor,
+        RecordConstructorArg, SrcSpan, TypeAlias, TypeAst, TypeAstConstructor, TypeAstFn,
+        TypeAstHole, TypeAstTuple, TypeAstVar, TypedDefinition, TypedModule, UntypedArg,
+        UntypedModule, UntypedStatement,
     },
     build::{Origin, Target},
     call_graph::{into_dependency_order, CallGraphNode},
+    config::PackageConfig,
     dep_tree,
+    line_numbers::LineNumbers,
     type_::{
         self,
         environment::*,
         error::{convert_unify_error, Error, MissingAnnotation},
-        expression::{ExprTyper, Implementations},
+        expression::{ExprTyper, FunctionDefinition},
         fields::{FieldMap, FieldMapBuilder},
         hydrator::Hydrator,
         prelude::*,
@@ -28,6 +31,7 @@ use crate::{
     warning::TypeWarningEmitter,
     GLEAM_CORE_PACKAGE_NAME,
 };
+use camino::Utf8PathBuf;
 use ecow::EcoString;
 use itertools::Itertools;
 use std::{
@@ -116,17 +120,20 @@ pub fn infer_module<A>(
     ids: &UniqueIdGenerator,
     mut module: UntypedModule,
     origin: Origin,
-    package: &EcoString,
     modules: &im::HashMap<EcoString, ModuleInterface>,
     warnings: &TypeWarningEmitter,
     direct_dependencies: &HashMap<EcoString, A>,
     target_support: TargetSupport,
+    line_numbers: LineNumbers,
+    package_config: &PackageConfig,
+    src_path: Utf8PathBuf,
 ) -> Result<TypedModule, Error> {
     let name = module.name.clone();
     let documentation = std::mem::take(&mut module.documentation);
+    let package = package_config.name.clone();
     let env = Environment::new(
         ids.clone(),
-        package.clone(),
+        package_config.name.clone(),
         name.clone(),
         target,
         modules,
@@ -150,7 +157,14 @@ pub fn infer_module<A>(
     // Register types so they can be used in constructors and functions
     // earlier in the module.
     for t in &statements.custom_types {
-        register_types_from_custom_type(t, &mut type_names, &mut env, &name, &mut hydrators)?;
+        register_types_from_custom_type(
+            t,
+            &mut type_names,
+            &mut env,
+            &name,
+            package_config,
+            &mut hydrators,
+        )?;
     }
     // TODO: Extract a Type alias class to perform this.
     let sorted_aliases = sorted_type_aliases(&statements.type_aliases)?;
@@ -181,7 +195,7 @@ pub fn infer_module<A>(
     for i in statements.imports {
         let statement = record_imported_items_for_use_detection(
             i,
-            package,
+            package.as_str(),
             direct_dependencies,
             warnings,
             &env,
@@ -235,11 +249,12 @@ pub fn infer_module<A>(
     // server, but are filtered out when type checking to prevent using private
     // items.
     env.module_types.retain(|_, info| info.module == name);
-    env.accessors.retain(|_, accessors| accessors.public);
+    env.accessors
+        .retain(|_, accessors| accessors.publicity.is_importable());
 
     // Ensure no exported values have private types in their type signature
     for value in env.module_values.values() {
-        if !value.public {
+        if value.publicity.is_private() {
             continue;
         }
         if let Some(leaked) = value.type_.find_private_type() {
@@ -248,6 +263,17 @@ pub fn infer_module<A>(
                 leaked,
             });
         }
+
+        // We also want to make sure that no public type exposes internal ones
+        // in their type signature.
+        if !value.publicity.is_public() {
+            continue;
+        }
+        // We also don't want to raise a warning if we're inside an internal
+        // module ourselves, the type wouldn't actually be publicly exposed.
+        if package_config.is_internal_module(name.as_str()) {
+            continue;
+        }
     }
 
     let Environment {
@@ -259,6 +285,8 @@ pub fn infer_module<A>(
         ..
     } = env;
 
+    let is_internal = package_config.is_internal_module(name.as_str());
+
     Ok(ast::Module {
         documentation,
         name: name.clone(),
@@ -270,9 +298,12 @@ pub fn infer_module<A>(
             values,
             accessors,
             origin,
-            package: package.clone(),
+            package: package_config.name.clone(),
+            is_internal,
             unused_imports,
             contains_todo,
+            line_numbers,
+            src_path,
         },
     })
 }
@@ -300,12 +331,13 @@ fn register_type_alias(
 ) -> Result<(), Error> {
     let TypeAlias {
         location,
-        public,
+        publicity,
         parameters: args,
         alias: name,
         type_ast: resolved_type,
         deprecation,
-        ..
+        type_: _,
+        documentation: _,
     } = t;
 
     // A type alias must not have the same name as any other type in the module.
@@ -324,10 +356,10 @@ fn register_type_alias(
         TypeConstructor {
             origin: *location,
             module: module.clone(),
-            public: *public,
             parameters,
             typ,
             deprecation: deprecation.clone(),
+            publicity: *publicity,
         },
     )?;
 
@@ -339,7 +371,7 @@ fn register_type_alias(
     }
 
     // Register the type for detection of dead code.
-    if !public {
+    if publicity.is_private() {
         environment.init_usage(name.clone(), EntityKind::PrivateType, *location);
     };
     Ok(())
@@ -350,14 +382,17 @@ fn register_types_from_custom_type<'a>(
     names: &mut HashMap<EcoString, SrcSpan>,
     environment: &mut Environment<'a>,
     module: &'a EcoString,
+    config: &PackageConfig,
     hydrators: &mut HashMap<EcoString, Hydrator>,
 ) -> Result<(), Error> {
     let CustomType {
         name,
-        public,
+        publicity,
         parameters,
         location,
         deprecation,
+        opaque,
+        constructors,
         ..
     } = t;
     assert_unique_type_name(names, name, *location)?;
@@ -366,8 +401,19 @@ fn register_types_from_custom_type<'a>(
 
     hydrator.clear_ridgid_type_names();
 
+    // We check is the type comes from an internal module and restrict its
+    // publicity.
+    let publicity = match publicity {
+        // It's important we only restrict the publicity of public types.
+        Publicity::Public if config.is_internal_module(module) => Publicity::Internal,
+        // If a type is private we don't want to make it internal just because
+        // it comes from an internal module, so in that case the publicity is
+        // left unchanged.
+        Publicity::Public | Publicity::Private | Publicity::Internal => *publicity,
+    };
+
     let typ = Arc::new(Type::Named {
-        public: *public,
+        publicity,
         package: environment.current_package.clone(),
         module: module.to_owned(),
         name: name.clone(),
@@ -379,13 +425,22 @@ fn register_types_from_custom_type<'a>(
         TypeConstructor {
             origin: *location,
             module: module.clone(),
-            public: *public,
             deprecation: deprecation.clone(),
             parameters,
+            publicity,
             typ,
         },
     )?;
-    if !public {
+
+    if *opaque && constructors.is_empty() {
+        environment
+            .warnings
+            .emit(type_::Warning::OpaqueExternalType {
+                location: *location,
+            });
+    }
+
+    if publicity.is_private() {
         environment.init_usage(name.clone(), EntityKind::PrivateType, *location);
     };
     Ok(())
@@ -401,13 +456,14 @@ fn register_values_from_custom_type(
 ) -> Result<(), Error> {
     let CustomType {
         location,
-        public,
+        publicity,
         opaque,
         name,
         constructors,
         deprecation,
         ..
     } = t;
+
     let mut hydrator = hydrators
         .remove(name)
         .expect("Could not find hydrator for register_values custom type");
@@ -420,7 +476,11 @@ fn register_values_from_custom_type(
         .clone();
     if let Some(accessors) = custom_type_accessors(constructors, &mut hydrator, environment)? {
         let map = AccessorsMap {
-            public: (*public && !*opaque),
+            publicity: if *opaque {
+                Publicity::Private
+            } else {
+                *publicity
+            },
             accessors,
             // TODO: improve the ownership here so that we can use the
             // `return_type_constructor` below rather than looking it up twice.
@@ -475,17 +535,25 @@ fn register_values_from_custom_type(
             constructor_index: index as u16,
         };
 
+        // If the contructor belongs to an opaque type then it's going to be
+        // considered as private.
+        let value_constructor_publicity = if *opaque {
+            Publicity::Private
+        } else {
+            *publicity
+        };
+
         environment.insert_module_value(
             constructor.name.clone(),
             ValueConstructor {
+                publicity: value_constructor_publicity,
                 deprecation: deprecation.clone(),
-                public: *public && !opaque,
                 type_: typ.clone(),
                 variant: constructor_info.clone(),
             },
         );
 
-        if !public {
+        if value_constructor_publicity.is_private() {
             environment.init_usage(
                 constructor.name.clone(),
                 EntityKind::PrivateTypeConstructor(name.clone()),
@@ -501,7 +569,7 @@ fn register_values_from_custom_type(
             constructor.name.clone(),
             constructor_info,
             typ,
-            *public,
+            value_constructor_publicity,
             deprecation.clone(),
         );
     }
@@ -527,7 +595,7 @@ fn register_value_from_function(
         arguments: args,
         location,
         return_annotation,
-        public,
+        publicity,
         documentation,
         external_erlang,
         external_javascript,
@@ -571,8 +639,8 @@ fn register_value_from_function(
         location: *location,
         implementations: *implementations,
     };
-    environment.insert_variable(name.clone(), variant, typ, *public, deprecation.clone());
-    if !public {
+    environment.insert_variable(name.clone(), variant, typ, *publicity, deprecation.clone());
+    if publicity.is_private() {
         environment.init_usage(name.clone(), EntityKind::PrivateFunction, *location);
     };
     Ok(())
@@ -625,7 +693,7 @@ fn infer_function(
         documentation: doc,
         location,
         name,
-        public,
+        publicity,
         arguments,
         body,
         return_annotation,
@@ -661,10 +729,10 @@ fn infer_function(
         ensure_annotations_present(&arguments, return_annotation.as_ref(), location)?;
     }
 
-    let external_implementations = Implementations {
-        gleam: false,
-        uses_erlang_externals: external_erlang.is_some(),
-        uses_javascript_externals: external_javascript.is_some(),
+    let definition = FunctionDefinition {
+        has_body: !body.first().is_placeholder(),
+        has_erlang_external: external_erlang.is_some(),
+        has_javascript_external: external_javascript.is_some(),
     };
 
     // Infer the type using the preregistered args + return types as a starting point
@@ -674,7 +742,7 @@ fn infer_function(
             .zip(&args_types)
             .map(|(a, t)| a.set_type(t.clone()))
             .collect();
-        let mut expr_typer = ExprTyper::new(environment, external_implementations);
+        let mut expr_typer = ExprTyper::new(environment, definition);
         expr_typer.hydrator = hydrators
             .remove(&name)
             .expect("Could not find hydrator for fn");
@@ -692,7 +760,10 @@ fn infer_function(
     // Ensure that the current target has an implementation for the function.
     // This is done at the expression level while inferring the function body, but we do it again
     // here as externally implemented functions may not have a Gleam body.
-    if public && environment.target_support.is_enforced() && !implementations.supports(target) {
+    if publicity.is_importable()
+        && environment.target_support.is_enforced()
+        && !implementations.supports(target)
+    {
         return Err(Error::UnsupportedPublicFunctionTarget {
             name: name.clone(),
             target,
@@ -714,7 +785,7 @@ fn infer_function(
         name.clone(),
         variant,
         type_.clone(),
-        public,
+        publicity,
         deprecation.clone(),
     );
 
@@ -722,7 +793,7 @@ fn infer_function(
         documentation: doc,
         location,
         name,
-        public,
+        publicity,
         deprecation,
         arguments: args,
         end_position: end_location,
@@ -737,7 +808,7 @@ fn infer_function(
     }))
 }
 
-/// Returns the the module name and function name of the implementation of a
+/// Returns the module name and function name of the implementation of a
 /// function. If the function is implemented as a Gleam function then it is the
 /// same as the name of the module and function. If the function has an external
 /// implementation then it is the name of the external module and function.
@@ -804,7 +875,7 @@ fn insert_type_alias(
     let TypeAlias {
         documentation: doc,
         location,
-        public,
+        publicity,
         alias,
         parameters: args,
         type_ast: resolved_type,
@@ -819,7 +890,7 @@ fn insert_type_alias(
     Ok(Definition::TypeAlias(TypeAlias {
         documentation: doc,
         location,
-        public,
+        publicity,
         alias,
         parameters: args,
         type_ast: resolved_type,
@@ -836,7 +907,7 @@ fn infer_custom_type(
         documentation: doc,
         location,
         end_position,
-        public,
+        publicity,
         opaque,
         name,
         parameters,
@@ -904,7 +975,7 @@ fn infer_custom_type(
         documentation: doc,
         location,
         end_position,
-        public,
+        publicity,
         opaque,
         name,
         parameters,
@@ -976,7 +1047,7 @@ fn infer_module_constant(
         location,
         name,
         annotation,
-        public,
+        publicity,
         value,
         deprecation,
         ..
@@ -984,10 +1055,10 @@ fn infer_module_constant(
 
     let mut expr_typer = ExprTyper::new(
         environment,
-        Implementations {
-            gleam: false,
-            uses_erlang_externals: false,
-            uses_javascript_externals: false,
+        FunctionDefinition {
+            has_body: true,
+            has_erlang_external: false,
+            has_javascript_external: false,
         },
     );
     let typed_expr = expr_typer.infer_const(&annotation, *value)?;
@@ -995,7 +1066,7 @@ fn infer_module_constant(
     let implementations = expr_typer.implementations;
 
     let variant = ValueConstructor {
-        public,
+        publicity,
         deprecation: deprecation.clone(),
         variant: ValueConstructorVariant::ModuleConstant {
             documentation: doc.clone(),
@@ -1011,12 +1082,12 @@ fn infer_module_constant(
         name.clone(),
         variant.variant.clone(),
         type_.clone(),
-        public,
+        publicity,
         Deprecation::NotDeprecated,
     );
     environment.insert_module_value(name.clone(), variant);
 
-    if !public {
+    if publicity.is_private() {
         environment.init_usage(name.clone(), EntityKind::PrivateConstant, location);
     }
 
@@ -1025,7 +1096,7 @@ fn infer_module_constant(
         location,
         name,
         annotation,
-        public,
+        publicity,
         value: Box::new(typed_expr),
         type_,
         deprecation,
@@ -1107,7 +1178,7 @@ fn generalise_module_constant(
         location,
         name,
         annotation,
-        public,
+        publicity,
         value,
         type_,
         deprecation,
@@ -1126,14 +1197,14 @@ fn generalise_module_constant(
         name.clone(),
         variant.clone(),
         type_.clone(),
-        public,
+        publicity,
         deprecation.clone(),
     );
 
     environment.insert_module_value(
         name.clone(),
         ValueConstructor {
-            public,
+            publicity,
             variant,
             deprecation: deprecation.clone(),
             type_: type_.clone(),
@@ -1145,7 +1216,7 @@ fn generalise_module_constant(
         location,
         name,
         annotation,
-        public,
+        publicity,
         value,
         type_,
         deprecation,
@@ -1162,7 +1233,7 @@ fn generalise_function(
         documentation: doc,
         location,
         name,
-        public,
+        publicity,
         deprecation,
         arguments: args,
         body,
@@ -1201,13 +1272,13 @@ fn generalise_function(
         name.clone(),
         variant.clone(),
         type_.clone(),
-        public,
+        publicity,
         deprecation.clone(),
     );
     environment.insert_module_value(
         name.clone(),
         ValueConstructor {
-            public,
+            publicity,
             deprecation: deprecation.clone(),
             type_,
             variant,
@@ -1218,7 +1289,7 @@ fn generalise_function(
         documentation: doc,
         location,
         name,
-        public,
+        publicity,
         deprecation,
         arguments: args,
         end_position: end_location,
