@@ -349,6 +349,43 @@ impl TypeAst {
     }
 }
 
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+pub enum Publicity {
+    Public,
+    Private,
+    Internal,
+}
+
+impl Publicity {
+    pub fn is_private(&self) -> bool {
+        match self {
+            Self::Private => true,
+            Self::Public | Self::Internal => false,
+        }
+    }
+
+    pub fn is_internal(&self) -> bool {
+        match self {
+            Self::Internal => true,
+            Self::Public | Self::Private => false,
+        }
+    }
+
+    pub fn is_public(&self) -> bool {
+        match self {
+            Self::Public => true,
+            Self::Internal | Self::Private => false,
+        }
+    }
+
+    pub fn is_importable(&self) -> bool {
+        match self {
+            Self::Internal | Self::Public => true,
+            Self::Private => false,
+        }
+    }
+}
+
 #[derive(Debug, Clone, PartialEq, Eq)]
 /// A function definition
 ///
@@ -366,7 +403,7 @@ pub struct Function<T, Expr> {
     pub name: EcoString,
     pub arguments: Vec<Arg<T>>,
     pub body: Vec1<Statement<T, Expr>>,
-    pub public: bool,
+    pub publicity: Publicity,
     pub deprecation: Deprecation,
     pub return_annotation: Option<TypeAst>,
     pub return_type: T,
@@ -436,7 +473,7 @@ pub type UntypedModuleConstant = ModuleConstant<(), ()>;
 pub struct ModuleConstant<T, ConstantRecordTag> {
     pub documentation: Option<EcoString>,
     pub location: SrcSpan,
-    pub public: bool,
+    pub publicity: Publicity,
     pub name: EcoString,
     pub annotation: Option<TypeAst>,
     pub value: Box<Constant<T, ConstantRecordTag>>,
@@ -467,7 +504,7 @@ pub struct CustomType<T> {
     pub location: SrcSpan,
     pub end_position: u32,
     pub name: EcoString,
-    pub public: bool,
+    pub publicity: Publicity,
     pub constructors: Vec<RecordConstructor<T>>,
     pub documentation: Option<EcoString>,
     pub deprecation: Deprecation,
@@ -505,7 +542,7 @@ pub struct TypeAlias<T> {
     pub parameters: Vec<EcoString>,
     pub type_ast: TypeAst,
     pub type_: T,
-    pub public: bool,
+    pub publicity: Publicity,
     pub documentation: Option<EcoString>,
     pub deprecation: Deprecation,
 }
@@ -527,6 +564,13 @@ pub enum Definition<T, Expr, ConstantRecordTag, PackageName> {
 }
 
 impl TypedDefinition {
+    pub fn main_function(&self) -> Option<&TypedFunction> {
+        match self {
+            Definition::Function(f) if f.name == "main" => Some(f),
+            _ => None,
+        }
+    }
+
     pub fn find_node(&self, byte_index: u32) -> Option<Located<'_>> {
         match self {
             Definition::Function(function) => {
@@ -629,6 +673,17 @@ impl<A, B, C, E> Definition<A, B, C, E> {
             }
         }
     }
+
+    pub fn is_internal(&self) -> bool {
+        match self {
+            Definition::Function(Function { publicity, .. })
+            | Definition::CustomType(CustomType { publicity, .. })
+            | Definition::ModuleConstant(ModuleConstant { publicity, .. })
+            | Definition::TypeAlias(TypeAlias { publicity, .. }) => publicity.is_internal(),
+
+            Definition::Import(_) => false,
+        }
+    }
 }
 
 #[derive(Debug, Clone, PartialEq, Eq)]
@@ -1336,6 +1391,13 @@ impl TypedPattern {
             return None;
         }
 
+        if let Pattern::Variable { name, .. } = self {
+            // For pipes the pattern can't be pointed to
+            if name.as_str().eq(PIPE_VARIABLE) {
+                return None;
+            }
+        }
+
         match self {
             Pattern::Int { .. }
             | Pattern::Float { .. }
@@ -1648,6 +1710,14 @@ impl UntypedStatement {
 }
 
 impl TypedStatement {
+    pub fn is_println(&self) -> bool {
+        match self {
+            Statement::Expression(e) => e.is_println(),
+            Statement::Assignment(_) => false,
+            Statement::Use(_) => false,
+        }
+    }
+
     pub fn is_non_pipe_expression(&self) -> bool {
         match self {
             Statement::Expression(expression) => !expression.is_pipeline(),
