@@ -2,7 +2,9 @@ use std::sync::Arc;
 
 use crate::analyse::TargetSupport;
 use crate::build::Target;
-use crate::type_::expression::Implementations;
+use crate::config::PackageConfig;
+use crate::line_numbers::LineNumbers;
+use crate::type_::expression::FunctionDefinition;
 use crate::type_::{Deprecation, PRELUDE_MODULE_NAME};
 use crate::{
     ast::{SrcSpan, TypedExpr},
@@ -15,29 +17,36 @@ use crate::{
     warning::TypeWarningEmitter,
 };
 
-use super::{Statement, TypedModule, TypedStatement};
+use super::{Publicity, Statement, TypedModule, TypedStatement};
 
 fn compile_module(src: &str) -> TypedModule {
     use crate::type_::build_prelude;
     let parsed = crate::parse::parse_module(src).expect("syntax error");
     let ast = parsed.module;
     let ids = UniqueIdGenerator::new();
+    let mut config = PackageConfig::default();
+    config.name = "thepackage".into();
     let mut modules = im::HashMap::new();
     // DUPE: preludeinsertion
     // TODO: Currently we do this here and also in the tests. It would be better
     // to have one place where we create all this required state for use in each
     // place.
     let _ = modules.insert(PRELUDE_MODULE_NAME.into(), build_prelude(&ids));
+    let line_numbers = LineNumbers::new(src);
+    let mut config = PackageConfig::default();
+    config.name = "thepackage".into();
     crate::analyse::infer_module::<()>(
         Target::Erlang,
         &ids,
         ast,
         crate::build::Origin::Src,
-        &"thepackage".into(),
         &modules,
         &TypeWarningEmitter::null(),
         &std::collections::HashMap::new(),
         TargetSupport::Enforced,
+        line_numbers,
+        &config,
+        "".into(),
     )
     .expect("should successfully infer")
 }
@@ -74,7 +83,7 @@ fn compile_expression(src: &str) -> TypedStatement {
 
     // Insert a cat record to use in the tests
     let cat_type = Arc::new(Type::Named {
-        public: true,
+        publicity: Publicity::Public,
         package: "mypackage".into(),
         module: "mymod".into(),
         name: "Cat".into(),
@@ -97,14 +106,14 @@ fn compile_expression(src: &str) -> TypedStatement {
         "Cat".into(),
         variant,
         type_::fn_(vec![type_::string(), type_::int()], cat_type.clone()),
-        true,
+        Publicity::Public,
         Deprecation::NotDeprecated,
     );
 
     environment.insert_accessors(
         "Cat".into(),
         AccessorsMap {
-            public: true,
+            publicity: Publicity::Public,
             type_: cat_type,
             accessors: [
                 (
@@ -129,10 +138,10 @@ fn compile_expression(src: &str) -> TypedStatement {
     );
     ExprTyper::new(
         &mut environment,
-        Implementations {
-            gleam: false,
-            uses_erlang_externals: false,
-            uses_javascript_externals: false,
+        FunctionDefinition {
+            has_body: true,
+            has_erlang_external: false,
+            has_javascript_external: false,
         },
     )
     .infer_statements(ast)
@@ -203,7 +212,7 @@ wibble}"#,
         location: SrcSpan { start: 16, end: 22 },
         constructor: ValueConstructor {
             deprecation: Deprecation::NotDeprecated,
-            public: false,
+            publicity: Publicity::Private,
             variant: ValueConstructorVariant::LocalVariable {
                 location: SrcSpan { start: 5, end: 11 },
             },
@@ -488,7 +497,7 @@ fn find_node_bool() {
         location: SrcSpan { start: 1, end: 5 },
         constructor: ValueConstructor {
             deprecation: Deprecation::NotDeprecated,
-            public: true,
+            publicity: Publicity::Public,
             variant: ValueConstructorVariant::Record {
                 documentation: None,
                 constructors_count: 2,
