@@ -1,4 +1,5 @@
 use crate::{
+    analyse::TargetSupport,
     build::{ErlangAppCodegenConfiguration, Module},
     config::PackageConfig,
     erlang,
@@ -122,29 +123,6 @@ impl<'a> ErlangApp<'a> {
                     .take_while(|_| self.config.include_dev_deps),
             )
             // TODO: test this!
-            // TODO: test this!
-            // TODO: test this!
-            // TODO: test this!
-            // TODO: test this!
-            // TODO: test this!
-            // TODO: test this!
-            // TODO: test this!
-            // TODO: test this!
-            // TODO: test this!
-            // TODO: test this!
-            // TODO: test this!
-            // TODO: test this!
-            // TODO: test this!
-            // TODO: test this!
-            // TODO: test this!
-            // TODO: test this!
-            // TODO: test this!
-            // TODO: test this!
-            // TODO: test this!
-            // TODO: test this!
-            // TODO: test this!
-            // TODO: test this!
-            // TODO: test this!
             .map(|name| self.config.package_name_overrides.get(name).unwrap_or(name))
             .chain(config.erlang.extra_applications.iter())
             .sorted()
@@ -182,6 +160,7 @@ pub struct JavaScript<'a> {
     output_directory: &'a Utf8Path,
     prelude_location: &'a Utf8Path,
     typescript: TypeScriptDeclarations,
+    target_support: TargetSupport,
 }
 
 impl<'a> JavaScript<'a> {
@@ -189,10 +168,12 @@ impl<'a> JavaScript<'a> {
         output_directory: &'a Utf8Path,
         typescript: TypeScriptDeclarations,
         prelude_location: &'a Utf8Path,
+        target_support: TargetSupport,
     ) -> Self {
         Self {
             prelude_location,
             output_directory,
+            target_support,
             typescript,
         }
     }
@@ -243,8 +224,14 @@ impl<'a> JavaScript<'a> {
         let name = format!("{js_name}.mjs");
         let path = self.output_directory.join(name);
         let line_numbers = LineNumbers::new(&module.code);
-        let output =
-            javascript::module(&module.ast, &line_numbers, &module.input_path, &module.code);
+        let output = javascript::module(
+            &module.ast,
+            &line_numbers,
+            &module.input_path,
+            &module.code,
+            self.target_support,
+            self.typescript,
+        );
         tracing::debug!(name = ?js_name, "Generated js module");
         writer.write(&path, &output?)
     }
