@@ -4,6 +4,7 @@ mod asignments;
 mod binary_operators;
 mod bit_array;
 mod blocks;
+mod cases;
 mod conditional_compilation;
 mod external_fn;
 mod external_types;
@@ -40,7 +41,7 @@ fn imports() {
     assert_format!("import one\n");
     assert_format!("import one\nimport two\n");
     assert_format!("import one/two/three\n");
-    assert_format!("import one/two/three\nimport four/five\n");
+    assert_format!("import four/five\nimport one/two/three\n");
     assert_format!("import one.{fun, fun2, fun3}\n");
     assert_format!("import one.{One, Two, fun1, fun2}\n");
     assert_format!("import one.{main as entrypoint}\n");
@@ -67,8 +68,8 @@ fn imports() {
 fn multiple_statements_test() {
     assert_format!(
         r#"import one
-import two
 import three
+import two
 
 pub type One
 
@@ -3731,8 +3732,8 @@ fn commented_binop() {
     assert_format!(
         "fn main() {
   1
-  + // hello
-  2
+  // hello
+  + 2
 }
 "
     );
@@ -3741,10 +3742,10 @@ fn commented_binop() {
         "fn main() {
   // one
   1
-  + // two
-  2
-  + // three
-  3
+  // two
+  + 2
+  // three
+  + 3
 }
 "
     );
@@ -4463,6 +4464,127 @@ fn negation_block() {
     );
 }
 
+#[test]
+fn empty_lines_work_with_trailing_space() {
+    let src = "pub fn main() {
+  let inc = fn(a) { a + 1 }
+
+
+  pair.map_first(#(1, 2), inc)
+  |> should.equal(#(2, 2))
+
+  // Comment
+
+  1
+
+
+  // Comment
+
+
+  2
+}
+";
+    let expected = "pub fn main() {
+  let inc = fn(a) { a + 1 }
+
+  pair.map_first(#(1, 2), inc)
+  |> should.equal(#(2, 2))
+
+  // Comment
+
+  1
+
+  // Comment
+
+  2
+}
+";
+    assert_format!(expected); // Sanity check
+
+    assert_format_rewrite!(src, expected);
+}
+
+#[test]
+fn empty_lines_work_with_eol_normalisation() {
+    let src = "pub fn main() {
+  let inc = fn(a) { a + 1 }
+
+
+  pair.map_first(#(1, 2), inc)
+  |> should.equal(#(2, 2))
+
+  // Comment
+
+  1
+
+
+  // Comment
+
+
+  2
+}
+";
+    let expected = "pub fn main() {
+  let inc = fn(a) { a + 1 }
+
+  pair.map_first(#(1, 2), inc)
+  |> should.equal(#(2, 2))
+
+  // Comment
+
+  1
+
+  // Comment
+
+  2
+}
+";
+    assert_format!(expected); // Sanity check
+
+    assert_format_rewrite!(&src.replace('\n', "\r\n"), expected);
+    assert_format_rewrite!(&src.replace('\n', "\r"), expected);
+}
+
+#[test]
+fn empty_lines_work_with_trailing_space_and_eol_normalisation() {
+    let src = "pub fn main() {
+  let inc = fn(a) { a + 1 }
+
+
+  pair.map_first(#(1, 2), inc)
+  |> should.equal(#(2, 2))
+
+  // Comment
+
+  1
+
+
+  // Comment
+
+
+  2
+}
+";
+    let expected = "pub fn main() {
+  let inc = fn(a) { a + 1 }
+
+  pair.map_first(#(1, 2), inc)
+  |> should.equal(#(2, 2))
+
+  // Comment
+
+  1
+
+  // Comment
+
+  2
+}
+";
+    assert_format!(expected); // Sanity check
+
+    assert_format_rewrite!(&src.replace('\n', "\r\n"), expected);
+    assert_format_rewrite!(&src.replace('\n', "\r"), expected);
+}
 #[test]
 fn single_empty_line_between_comments() {
     // empty line isn't added if it's not already present
@@ -5549,3 +5671,281 @@ fn pipeline_inside_tuple_is_not_nested_if_only_item() {
 "#
     );
 }
+
+// github.com/gleam-lang/gleam/issues/2608
+#[test]
+fn comments_are_not_moved_out_of_list_of_literals() {
+    assert_format!(
+        r#"fn main() {
+  [
+    1, 2,
+    // list
+  ]
+}
+"#
+    );
+}
+
+// github.com/gleam-lang/gleam/issues/2608
+#[test]
+fn comments_are_not_moved_out_of_list() {
+    assert_format!(
+        r#"fn main() {
+  [
+    wibble,
+    wobble,
+    // list
+  ]
+}
+"#
+    );
+}
+
+// github.com/gleam-lang/gleam/issues/2608
+#[test]
+fn comments_are_not_moved_out_of_case_expressions() {
+    assert_format!(
+        r#"fn main() {
+  case True {
+    _ -> Nil
+    // case
+  }
+}
+"#
+    );
+}
+
+// github.com/gleam-lang/gleam/issues/2608
+#[test]
+fn comments_are_not_moved_out_of_tuples() {
+    assert_format!(
+        r#"fn main() {
+  #(
+    1,
+    2,
+    // tuple
+  )
+}
+"#
+    );
+}
+
+// github.com/gleam-lang/gleam/issues/2608
+#[test]
+fn comments_are_not_moved_out_of_function_calls() {
+    assert_format!(
+        r#"fn main() {
+  call(
+    1,
+    2,
+    // function call
+  )
+}
+"#
+    );
+}
+
+// https://github.com/gleam-lang/gleam/issues/2607
+#[test]
+fn function_arguments_after_comment_are_not_indented() {
+    assert_format!(
+        r#"pub fn main() {
+  wibble(
+    // Wobble
+    1 + 1,
+    "wibble",
+  )
+}
+"#
+    );
+}
+
+// https://github.com/gleam-lang/gleam/issues/2607
+#[test]
+fn tuple_items_after_comment_are_not_indented() {
+    assert_format!(
+        r#"pub fn main() {
+  #(
+    // Wobble
+    1 + 1,
+    "wibble",
+  )
+}
+"#
+    );
+}
+
+// https://github.com/gleam-lang/gleam/issues/2607
+#[test]
+fn list_items_after_comment_are_not_indented() {
+    assert_format!(
+        r#"pub fn main() {
+  [
+    // Wobble
+    1 + 1,
+    "wibble",
+  ]
+}
+"#
+    );
+}
+
+// https://github.com/gleam-lang/gleam/issues/2990
+#[test]
+fn comments_are_not_moved_out_of_empty_list() {
+    assert_format!(
+        r#"pub fn main() {
+  // This is an empty list!
+  [
+    // Nothing here...
+  ]
+}
+"#
+    );
+}
+
+#[test]
+fn empty_lists_with_comment_inside_are_indented_properly() {
+    assert_format!(
+        r#"pub fn main() {
+  fun(
+    [
+      // Nothing here...
+    ],
+    wibble_wobble_wibble_wobble_wibble_wobble_wibble_wobble,
+    [
+      // Nothing here as well!
+    ],
+  )
+}
+"#
+    );
+}
+
+// https://github.com/gleam-lang/gleam/issues/2890
+#[test]
+fn piped_blocks_are_not_needlessly_indented() {
+    assert_format!(
+        r#"pub fn main() {
+  #(
+    1,
+    {
+      "long enough to need to wrap. blah blah blah blah blah blah blah blah blah"
+    }
+      |> foo,
+    3,
+  )
+}
+"#
+    );
+}
+
+// https://github.com/gleam-lang/gleam/issues/2924
+#[test]
+fn record_update_fields_are_not_needlessly_broken() {
+    assert_format!(
+        r#"pub fn main() {
+  Model(
+    ..model,
+    wibble: wibble_wobble_wibble_wobble + 1,
+    wobble: Some(wibble_wobble_wibble_wobble),
+  )
+}
+"#
+    );
+}
+
+// https://github.com/gleam-lang/gleam/issues/2890
+#[test]
+fn piped_lists_are_not_needlessly_indented() {
+    assert_format!(
+        r#"pub fn main() {
+  fun(
+    [
+      ["wibble wobble", "wibble", "wobble"],
+      ["long enough to go over", "line limit"],
+    ]
+      |> list.concat,
+    todo,
+  )
+}
+"#
+    );
+}
+
+#[test]
+fn comments_inside_nested_pipe_chain() {
+    assert_format!(
+        r#"pub fn main() {
+  fun(
+    thing
+      // A comment
+      |> wibble
+      // Another comment
+      |> wobble,
+    thing,
+  )
+}
+"#
+    );
+}
+
+#[test]
+fn comments_inside_nested_binop_chain() {
+    assert_format!(
+        r#"pub fn main() {
+  fun(
+    thing
+      // A comment
+      <> wibble
+      // Another comment
+      <> wobble,
+    thing,
+  )
+}
+"#
+    );
+}
+
+#[test]
+fn comments_inside_binop_chain() {
+    assert_format!(
+        r#"pub fn main() {
+  thing
+  // A comment
+  <> wibble
+  // Another comment
+  <> wobble
+}
+"#
+    );
+}
+
+#[test]
+fn internal_attribute_on_function() {
+    assert_format!(
+        r#"@internal
+pub fn main() {
+  todo
+}
+"#
+    );
+}
+
+#[test]
+fn internal_attribute_on_type() {
+    assert_format!(
+        r#"@internal
+pub type Type
+"#
+    );
+}
+
+#[test]
+fn internal_attribute_on_const() {
+    assert_format!(
+        r#"@internal
+pub const wibble = 1
+"#
+    );
+}
