@@ -16,6 +16,15 @@ use camino::{Utf8Path, Utf8PathBuf};
 /// The provided source path should be absolute, otherwise will panic.
 pub fn make_relative(source_path: &Utf8Path, target_path: &Utf8Path) -> Utf8PathBuf {
     assert!(source_path.is_absolute());
+    // Input target will always be canonicalised whereas source will not
+    // This causes problems with diffing on windows since canonicalised paths have a special root
+    // As such we are attempting to strip the target path
+    // Based on https://github.com/rust-lang/rust/issues/42869#issuecomment-1712317081
+    #[cfg(target_family = "windows")]
+    let binding = target_path.to_string();
+    #[cfg(target_family = "windows")]
+    let target_path = Utf8Path::new(binding.trim_start_matches(r"\\?\"));
+
     match target_path.is_absolute() {
         true => pathdiff::diff_utf8_paths(target_path, source_path)
             .expect("Should not fail on two absolute paths"),
