@@ -141,16 +141,17 @@ impl<'module> Generator<'module> {
             TypedExpr::Int { value, .. } => Ok(int(value)),
             TypedExpr::Float { value, .. } => Ok(float(value)),
 
-            TypedExpr::List { elements, tail, .. } => {
-                self.tracker.list_used = true;
-                self.not_in_tail_position(|gen| {
-                    let tail = match tail {
-                        Some(tail) => Some(gen.wrap_expression(tail)?),
-                        None => None,
-                    };
-                    list(elements.iter().map(|e| gen.wrap_expression(e)), tail)
-                })
-            }
+            TypedExpr::List { elements, tail, .. } => self.not_in_tail_position(|gen| match tail {
+                Some(tail) => {
+                    gen.tracker.prepend_used = true;
+                    let tail = gen.wrap_expression(tail)?;
+                    prepend(elements.iter().map(|e| gen.wrap_expression(e)), tail)
+                }
+                None => {
+                    gen.tracker.list_used = true;
+                    list(elements.iter().map(|e| gen.wrap_expression(e)))
+                }
+            }),
 
             TypedExpr::Tuple { elems, .. } => self.tuple(elems),
             TypedExpr::TupleIndex { tuple, index, .. } => self.tuple_index(tuple, *index),
@@ -264,7 +265,7 @@ impl<'module> Generator<'module> {
         Ok(docvec!["toBitArray(", segments_array, ")"])
     }
 
-    pub fn wrap_return<'a>(&self, document: Document<'a>) -> Document<'a> {
+    pub fn wrap_return<'a>(&mut self, document: Document<'a>) -> Document<'a> {
         if self.scope_position.is_tail() {
             docvec!["return ", document, ";"]
         } else {
@@ -308,14 +309,19 @@ impl<'module> Generator<'module> {
     /// being an operator or a function literal.
     pub fn child_expression<'a>(&mut self, expression: &'a TypedExpr) -> Output<'a> {
         match expression {
-            TypedExpr::BinOp { name, .. } if name.is_operator_to_wrap() => {
-                Ok(docvec!("(", self.expression(expression)?, ")"))
-            }
+            TypedExpr::BinOp { name, .. } if name.is_operator_to_wrap() => {}
+            TypedExpr::Fn { .. } => {}
 
-            TypedExpr::Fn { .. } => Ok(docvec!("(", self.expression(expression)?, ")")),
-
-            _ => self.wrap_expression(expression),
+            _ => return self.wrap_expression(expression),
         }
+
+        let document = self.expression(expression)?;
+        Ok(if self.scope_position.is_tail() {
+            // Here the document is a return statement: `return <expr>;`
+            document
+        } else {
+            docvec!("(", document, ")")
+        })
     }
 
     /// Wrap an expression in an immediately involked function expression
@@ -342,7 +348,8 @@ impl<'module> Generator<'module> {
         self.scope_position = scope_position;
 
         // Wrap in iife document
-        Ok(self.immediately_involked_function_expression_document(result?))
+        let doc = self.immediately_involked_function_expression_document(result?);
+        Ok(self.wrap_return(doc))
     }
 
     /// Wrap a document in an immediately involked function expression
@@ -958,7 +965,7 @@ impl<'module> Generator<'module> {
             Some(m) => self.expression(m)?,
             None => string("panic expression evaluated"),
         };
-        let doc = self.throw_error("todo", &message, *location, vec![]);
+        let doc = self.throw_error("panic", &message, *location, vec![]);
 
         // Reset tail position so later values are returned as needed. i.e.
         // following clauses in a case expression.
@@ -1066,7 +1073,7 @@ impl<'module> Generator<'module> {
 
     fn pattern_assignments_doc(assignments: Vec<Assignment<'_>>) -> Document<'_> {
         let assignments = assignments.into_iter().map(pattern::Assignment::into_doc);
-        concat(Itertools::intersperse(assignments, line()))
+        join(assignments, line())
     }
 
     fn pattern_take_assignments_doc<'a>(
@@ -1100,12 +1107,17 @@ impl<'module> Generator<'module> {
             break_(" ||", " || ")
         };
 
-        concat(Itertools::intersperse(
-            checks
-                .into_iter()
-                .map(|check| check.into_doc(match_desired)),
+        let checks_len = checks.len();
+        join(
+            checks.into_iter().map(|check| {
+                if checks_len > 1 && check.may_require_wrapping() {
+                    docvec!["(", check.into_doc(match_desired), ")"]
+                } else {
+                    check.into_doc(match_desired)
+                }
+            }),
             operator,
-        ))
+        )
         .group()
     }
 }
@@ -1179,7 +1191,6 @@ pub(crate) fn guard_constant_expression<'a>(
                 elements
                     .iter()
                     .map(|e| guard_constant_expression(assignments, tracker, e)),
-                None,
             )
         }
         Constant::Record { typ, name, .. } if typ.is_bool() && name == "True" => {
@@ -1211,11 +1222,16 @@ pub(crate) fn guard_constant_expression<'a>(
             Ok(construct_record(module.as_deref(), tag, field_values))
         }
 
+        Constant::BitArray { segments, .. } => bit_array(tracker, segments, |tracker, constant| {
+            guard_constant_expression(assignments, tracker, constant)
+        }),
+
         Constant::Var { name, .. } => Ok(assignments
             .iter()
             .find(|assignment| assignment.name == name)
             .map(|assignment| assignment.subject.clone().append(assignment.path.clone()))
             .unwrap_or_else(|| name.to_doc())),
+
         expression => constant_expression(tracker, expression),
     }
 }
@@ -1234,10 +1250,7 @@ pub(crate) fn constant_expression<'a>(
 
         Constant::List { elements, .. } => {
             tracker.list_used = true;
-            list(
-                elements.iter().map(|e| constant_expression(tracker, e)),
-                None,
-            )
+            list(elements.iter().map(|e| constant_expression(tracker, e)))
         }
 
         Constant::Record { typ, name, .. } if typ.is_bool() && name == "True" => {
@@ -1269,7 +1282,7 @@ pub(crate) fn constant_expression<'a>(
             Ok(construct_record(module.as_deref(), tag, field_values))
         }
 
-        Constant::BitArray { segments, .. } => bit_array(tracker, segments),
+        Constant::BitArray { segments, .. } => bit_array(tracker, segments, constant_expression),
 
         Constant::Var { name, module, .. } => Ok({
             match module {
@@ -1283,13 +1296,14 @@ pub(crate) fn constant_expression<'a>(
 fn bit_array<'a>(
     tracker: &mut UsageTracker,
     segments: &'a [BitArraySegment<TypedConstant, Arc<Type>>],
-) -> Result<Document<'a>, Error> {
+    mut constant_expr_fun: impl FnMut(&mut UsageTracker, &'a TypedConstant) -> Output<'a>,
+) -> Output<'a> {
     tracker.bit_array_literal_used = true;
 
     use BitArrayOption as Opt;
 
     let segments_array = array(segments.iter().map(|segment| {
-        let value = constant_expression(tracker, &segment.value)?;
+        let value = constant_expr_fun(tracker, &segment.value)?;
         match segment.options.as_slice() {
             // Ints
             [] | [Opt::Int { .. }] => Ok(value),
@@ -1297,7 +1311,7 @@ fn bit_array<'a>(
             // Sized ints
             [Opt::Size { value: size, .. }] => {
                 tracker.sized_integer_segment_used = true;
-                let size = constant_expression(tracker, size)?;
+                let size = constant_expr_fun(tracker, size)?;
                 Ok(docvec!["sizedInt(", value, ", ", size, ")"])
             }
 
@@ -1343,31 +1357,37 @@ pub fn string(value: &str) -> Document<'_> {
 
 pub fn array<'a, Elements: IntoIterator<Item = Output<'a>>>(elements: Elements) -> Output<'a> {
     let elements = Itertools::intersperse(elements.into_iter(), Ok(break_(",", ", ")))
-        .collect::<Result<Vec<_>, _>>()?
-        .to_doc();
-    Ok(docvec![
-        "[",
-        docvec![break_("", ""), elements].nest(INDENT),
-        break_(",", ""),
-        "]"
-    ]
-    .group())
+        .collect::<Result<Vec<_>, _>>()?;
+    if elements.is_empty() {
+        // Do not add a trailing comma since that adds an 'undefined' element
+        Ok("[]".to_doc())
+    } else {
+        Ok(docvec![
+            "[",
+            docvec![break_("", ""), elements].nest(INDENT),
+            break_(",", ""),
+            "]"
+        ]
+        .group())
+    }
 }
 
-fn list<'a, I: IntoIterator<Item = Output<'a>>>(
-    elements: I,
-    tail: Option<Document<'a>>,
-) -> Output<'a>
+fn list<'a, I: IntoIterator<Item = Output<'a>>>(elements: I) -> Output<'a>
 where
     I::IntoIter: DoubleEndedIterator + ExactSizeIterator,
 {
     let array = array(elements);
-    if let Some(tail) = tail {
-        let args = [array, Ok(tail)];
-        Ok(docvec!["toList", call_arguments(args)?])
-    } else {
-        Ok(docvec!["toList(", array?, ")"])
-    }
+    Ok(docvec!["toList(", array?, ")"])
+}
+
+fn prepend<'a, I: IntoIterator<Item = Output<'a>>>(elements: I, tail: Document<'a>) -> Output<'a>
+where
+    I::IntoIter: DoubleEndedIterator + ExactSizeIterator,
+{
+    elements.into_iter().rev().try_fold(tail, |tail, element| {
+        let args = call_arguments([element, Ok(tail)])?;
+        Ok(docvec!["listPrepend", args])
+    })
 }
 
 fn call_arguments<'a, Elements: IntoIterator<Item = Output<'a>>>(elements: Elements) -> Output<'a> {
@@ -1392,13 +1412,13 @@ fn construct_record<'a>(
     arguments: impl IntoIterator<Item = Document<'a>>,
 ) -> Document<'a> {
     let mut any_arguments = false;
-    let arguments = concat(Itertools::intersperse(
+    let arguments = join(
         arguments.into_iter().map(|a| {
             any_arguments = true;
             a
         }),
         break_(",", ", "),
-    ));
+    );
     let arguments = docvec![break_("", ""), arguments].nest(INDENT);
     let name = if let Some(module) = module {
         docvec!["$", module, ".", name]
@@ -1492,14 +1512,14 @@ fn requires_semicolon(statement: &TypedStatement) -> bool {
             | TypedExpr::NegateBool { .. }
             | TypedExpr::RecordUpdate { .. }
             | TypedExpr::RecordAccess { .. }
-            | TypedExpr::ModuleSelect { .. },
+            | TypedExpr::ModuleSelect { .. }
+            | TypedExpr::Block { .. },
         ) => true,
 
         Statement::Expression(
             TypedExpr::Todo { .. }
             | TypedExpr::Case { .. }
             | TypedExpr::Panic { .. }
-            | TypedExpr::Block { .. }
             | TypedExpr::Pipeline { .. },
         ) => false,
 
