@@ -1,6 +1,7 @@
 use crate::{
     analyse::TargetSupport,
     build::{Origin, Target},
+    config::PackageConfig,
     javascript::*,
     uid::UniqueIdGenerator,
     warning::TypeWarningEmitter,
@@ -95,19 +96,24 @@ pub fn compile(src: &str, deps: Vec<(&str, &str, &str)>) -> TypedModule {
     let mut direct_dependencies = std::collections::HashMap::from_iter(vec![]);
 
     deps.iter().for_each(|(dep_package, dep_name, dep_src)| {
+        let mut dep_config = PackageConfig::default();
+        dep_config.name = (*dep_package).into();
         let parsed = crate::parse::parse_module(dep_src).expect("dep syntax error");
         let mut ast = parsed.module;
         ast.name = (*dep_name).into();
+        let line_numbers = LineNumbers::new(dep_src);
         let dep = crate::analyse::infer_module::<()>(
             Target::JavaScript,
             &ids,
             ast,
             Origin::Src,
-            &((*dep_package).into()),
             &modules,
             &TypeWarningEmitter::null(),
             &std::collections::HashMap::new(),
             TargetSupport::Enforced,
+            line_numbers,
+            &dep_config,
+            "".into(),
         )
         .expect("should successfully infer");
         let _ = modules.insert((*dep_name).into(), dep.type_info);
@@ -117,16 +123,22 @@ pub fn compile(src: &str, deps: Vec<(&str, &str, &str)>) -> TypedModule {
     let parsed = crate::parse::parse_module(src).expect("syntax error");
     let mut ast = parsed.module;
     ast.name = "my/mod".into();
+    let line_numbers = LineNumbers::new(src);
+    let mut config = PackageConfig::default();
+    config.name = "thepackage".into();
+
     crate::analyse::infer_module::<()>(
         Target::JavaScript,
         &ids,
         ast,
         Origin::Src,
-        &"thepackage".into(),
         &modules,
         &TypeWarningEmitter::null(),
         &direct_dependencies,
-        TargetSupport::Enforced,
+        TargetSupport::NotEnforced,
+        line_numbers,
+        &config,
+        "".into(),
     )
     .expect("should successfully infer")
 }
@@ -134,7 +146,15 @@ pub fn compile(src: &str, deps: Vec<(&str, &str, &str)>) -> TypedModule {
 pub fn compile_js(src: &str, deps: Vec<(&str, &str, &str)>) -> String {
     let ast = compile(src, deps);
     let line_numbers = LineNumbers::new(src);
-    module(&ast, &line_numbers, Utf8Path::new(""), &"".into()).unwrap()
+    module(
+        &ast,
+        &line_numbers,
+        Utf8Path::new(""),
+        &"".into(),
+        TargetSupport::NotEnforced,
+        TypeScriptDeclarations::None,
+    )
+    .unwrap()
 }
 
 pub fn compile_ts(src: &str, deps: Vec<(&str, &str, &str)>) -> String {
