@@ -258,3 +258,49 @@ pub fn one(x: Int) -> Int {
 "#
     );
 }
+
+#[test]
+fn erlang_only() {
+    assert_js!(
+        r#"
+pub fn should_be_generated(x: Int) -> Int {
+  x
+}
+
+@external(erlang, "one", "one")
+pub fn should_not_be_generated(x: Int) -> Int
+"#
+    );
+}
+
+#[test]
+fn erlang_bit_patterns() {
+    assert_js!(
+        r#"
+pub fn should_not_be_generated(x) {
+  case x {
+    <<_, rest:bits>> -> rest
+    _ -> x
+  }
+}
+"#
+    );
+}
+
+#[test]
+fn both_externals_no_valid_impl() {
+    assert_js!(
+        r#"
+@external(javascript, "one", "one")
+pub fn js() -> Nil
+
+@external(erlang, "one", "one")
+pub fn erl() -> Nil
+
+pub fn should_not_be_generated() {
+  js()
+  erl()
+}
+"#
+    );
+}
