@@ -1,6 +1,7 @@
 use crate::{
     ast::{
-        Arg, Definition, Function, Import, ModuleConstant, TypedDefinition, TypedExpr, TypedPattern,
+        Arg, Definition, Function, Import, ModuleConstant, Publicity, TypedDefinition, TypedExpr,
+        TypedPattern,
     },
     build::{Located, Module},
     config::PackageConfig,
@@ -45,7 +46,7 @@ pub struct LanguageServerEngine<IO, Reporter> {
 
     /// A compiler for the project that supports repeat compilation of the root
     /// package.
-    /// In the event the the project config changes this will need to be
+    /// In the event the project config changes this will need to be
     /// discarded and reloaded to handle any changes to dependencies.
     pub(crate) compiler: LspProjectCompiler<FileSystemProxy<IO>>,
 
@@ -138,22 +139,9 @@ where
         self.compiler.take_warnings()
     }
 
-    // TODO: test local variables
-    // TODO: test same module constants
-    // TODO: test imported module constants
-    // TODO: test unqualified imported module constants
-    // TODO: test same module records
-    // TODO: test imported module records
-    // TODO: test unqualified imported module records
-    // TODO: test same module functions
-    // TODO: test module function calls
     // TODO: test different package module function calls
     //
-    //
-    //
     // TODO: implement unqualified imported module functions
-    // TODO: implement goto definition of modules that do not belong to the top
-    // level package.
     //
     pub fn goto_definition(
         &mut self,
@@ -176,11 +164,7 @@ where
                 Some(name) => {
                     let module = match this.compiler.get_source(name) {
                         Some(module) => module,
-                        // TODO: support goto definition for functions defined in
-                        // different packages. Currently it is not possible as the
-                        // required LineNumbers and source file path information is
-                        // not stored in the module metadata.
-                        None => return Ok(None),
+                        _ => return Ok(None),
                     };
                     let url = Url::parse(&format!("file:///{}", &module.path))
                         .expect("goto definition URL parse");
@@ -196,6 +180,7 @@ where
     pub fn completion(
         &mut self,
         params: lsp::TextDocumentPositionParams,
+        src: EcoString,
     ) -> Response<Option<Vec<lsp::CompletionItem>>> {
         self.respond(|this| {
             let module = match this.module_for_uri(&params.text_document.uri) {
@@ -203,6 +188,13 @@ where
                 None => return Ok(None),
             };
 
+            // Check current filercontents if the user is writing an import
+            // and handle separately from the rest of the completion flow
+            // Check if an import is being written
+            if let Some(value) = this.import_completions(src, &params, module) {
+                return value;
+            }
+
             let line_numbers = LineNumbers::new(&module.code);
             let byte_index =
                 line_numbers.byte_index(params.position.line, params.position.character);
@@ -382,8 +374,16 @@ where
 
             // Qualified types
             for (name, type_) in &module.types {
-                if !type_.public {
-                    continue;
+                match type_.publicity {
+                    // We skip private types as we never want those to appear in
+                    // completions.
+                    Publicity::Private => continue,
+                    // We only skip internal types if those are not defined in
+                    // the root package.
+                    Publicity::Internal if module.package != self.root_package_name() => continue,
+                    Publicity::Internal => {}
+                    // We never skip public types.
+                    Publicity::Public => {}
                 }
 
                 let module = import.used_name();
@@ -394,10 +394,12 @@ where
 
             // Unqualified types
             for unqualified in &import.unqualified_types {
-                let Some(type_) = module.get_public_type(&unqualified.name) else {
-                    continue;
-                };
-                completions.push(type_completion(None, unqualified.used_name(), type_));
+                match module.get_public_type(&unqualified.name) {
+                    Some(type_) => {
+                        completions.push(type_completion(None, unqualified.used_name(), type_))
+                    }
+                    None => continue,
+                }
             }
         }
 
@@ -409,6 +411,9 @@ where
 
         // Module functions
         for (name, value) in &module.ast.type_info.values {
+            // Here we do not check for the internal attribute: we always want
+            // to show autocompletions for values defined in the same module,
+            // even if those are internal.
             completions.push(value_completion(None, name, value));
         }
 
@@ -423,8 +428,16 @@ where
 
             // Qualified values
             for (name, value) in &module.values {
-                if !value.public {
-                    continue;
+                match value.publicity {
+                    // We skip private values as we never want those to appear in
+                    // completions.
+                    Publicity::Private => continue,
+                    // We only skip internal values if those are not defined in
+                    // the root package.
+                    Publicity::Internal if module.package != self.root_package_name() => continue,
+                    Publicity::Internal => {}
+                    // We never skip public values.
+                    Publicity::Public => {}
                 }
 
                 let module = import.used_name();
@@ -435,15 +448,113 @@ where
 
             // Unqualified values
             for unqualified in &import.unqualified_values {
-                let Some(value) = module.get_public_value(&unqualified.name) else {
-                    continue;
-                };
-                completions.push(value_completion(None, unqualified.used_name(), value));
+                match module.get_public_value(&unqualified.name) {
+                    Some(value) => {
+                        completions.push(value_completion(None, unqualified.used_name(), value))
+                    }
+                    None => continue,
+                }
             }
         }
 
         completions
     }
+
+    fn import_completions<'b>(
+        &'b self,
+        src: EcoString,
+        params: &lsp::TextDocumentPositionParams,
+        module: &'b Module,
+    ) -> Option<Result<Option<Vec<lsp::CompletionItem>>>> {
+        let line_num = LineNumbers::new(src.as_str());
+        let start_of_line = line_num.byte_index(params.position.line, 0);
+        let end_of_line = line_num.byte_index(params.position.line + 1, 0);
+
+        // Drop all lines before the line the cursor is on
+        let src = &src.get(start_of_line as usize..)?;
+
+        // If this isn't an import line then we don't offer import completions
+        if !src.trim_start().starts_with("import") {
+            return None;
+        }
+
+        // Find where to start and end the import completion
+        let start = line_num.line_and_column_number(start_of_line);
+        let end = line_num.line_and_column_number(end_of_line - 1);
+        let start = lsp::Position::new(start.line - 1, start.column + 6);
+        let end = lsp::Position::new(end.line - 1, end.column);
+        let completions = self.complete_modules_for_import(module, start, end);
+
+        Some(Ok(Some(completions)))
+    }
+
+    fn complete_modules_for_import<'b>(
+        &'b self,
+        current_module: &'b Module,
+        start: lsp::Position,
+        end: lsp::Position,
+    ) -> Vec<lsp::CompletionItem> {
+        let mut direct_dep_packages: std::collections::HashSet<&EcoString> =
+            std::collections::HashSet::from_iter(
+                self.compiler.project_compiler.config.dependencies.keys(),
+            );
+        if !current_module.origin.is_src() {
+            // In tests we can import direct dev dependencies
+            direct_dep_packages.extend(
+                self.compiler
+                    .project_compiler
+                    .config
+                    .dev_dependencies
+                    .keys(),
+            )
+        }
+
+        let already_imported: std::collections::HashSet<EcoString> =
+            std::collections::HashSet::from_iter(current_module.dependencies_list());
+        self.compiler
+            .project_compiler
+            .get_importable_modules()
+            .iter()
+            //
+            // It is possible to import modules from dependencies of dependencies
+            // but it's not recommended so we don't include them in completions
+            .filter(|(_, module)| {
+                let is_root_or_prelude =
+                    module.package == self.root_package_name() || module.package.is_empty();
+                is_root_or_prelude || direct_dep_packages.contains(&module.package)
+            })
+            //
+            // src/ cannot import test/
+            .filter(|(_, module)| module.origin.is_src() || !current_module.origin.is_src())
+            //
+            // It is possible to import internal modules from other packages,
+            // but it's not recommended so we don't include them in completions
+            .filter(|(_, module)| module.package == self.root_package_name() || !module.is_internal)
+            //
+            // You cannot import a module twice
+            .filter(|(name, _)| !already_imported.contains(*name))
+            //
+            // You cannot import yourself
+            .filter(|(name, _)| *name != &current_module.name)
+            //
+            // Everything else we suggest as a completion
+            .map(|(name, _)| lsp::CompletionItem {
+                label: name.to_string(),
+                kind: Some(lsp::CompletionItemKind::MODULE),
+                text_edit: {
+                    Some(lsp::CompletionTextEdit::Edit(lsp::TextEdit {
+                        range: lsp::Range { start, end },
+                        new_text: name.to_string(),
+                    }))
+                },
+                ..Default::default()
+            })
+            .collect()
+    }
+
+    fn root_package_name(&self) -> &str {
+        self.compiler.project_compiler.config.name.as_str()
+    }
 }
 
 fn type_completion(
@@ -657,7 +768,7 @@ fn get_expr_qualified_name(expression: &TypedExpr) -> Option<(&EcoString, &EcoSt
     match expression {
         TypedExpr::Var {
             name, constructor, ..
-        } if constructor.public => match &constructor.variant {
+        } if constructor.publicity.is_importable() => match &constructor.variant {
             ValueConstructorVariant::ModuleFn {
                 module: module_name,
                 ..
