@@ -2,7 +2,7 @@ use ecow::EcoString;
 
 use crate::{
     ast::{
-        Constant, SrcSpan, TypedConstant, TypedConstantBitArraySegment,
+        Constant, Publicity, SrcSpan, TypedConstant, TypedConstantBitArraySegment,
         TypedConstantBitArraySegmentOption,
     },
     schema_capnp::{self as schema, *},
@@ -41,16 +41,29 @@ impl<'a> ModuleEncoder<'a> {
         module.set_name(&self.data.name);
         module.set_package(&self.data.package);
         module.set_contains_todo(self.data.contains_todo);
+        module.set_src_path(self.data.src_path.as_str());
+        module.set_is_internal(self.data.is_internal);
         self.set_module_types(&mut module);
         self.set_module_values(&mut module);
         self.set_module_accessors(&mut module);
         self.set_module_types_constructors(&mut module);
         self.set_unused_imports(&mut module);
+        self.set_line_numbers(&mut module);
 
         capnp::serialize_packed::write_message(&mut buffer, &message).expect("capnp encode");
         Ok(buffer)
     }
 
+    fn set_line_numbers(&mut self, module: &mut module::Builder<'_>) {
+        let mut line_numbers = module.reborrow().init_line_numbers();
+        line_numbers.set_length(self.data.line_numbers.length);
+        let line_starts =
+            line_numbers.init_line_starts(self.data.line_numbers.line_starts.len() as u32);
+        for (i, l) in self.data.line_numbers.line_starts.iter().enumerate() {
+            line_starts.reborrow().set(i as u32, *l);
+        }
+    }
+
     fn set_unused_imports(&mut self, module: &mut module::Builder<'_>) {
         let mut unused_imports = module
             .reborrow()
@@ -154,12 +167,12 @@ impl<'a> ModuleEncoder<'a> {
         mut builder: type_constructor::Builder<'_>,
         constructor: &TypeConstructor,
     ) {
-        builder.set_public(constructor.public);
         builder.set_module(&constructor.module);
         builder.set_deprecated(match &constructor.deprecation {
             Deprecation::NotDeprecated => "",
             Deprecation::Deprecated { message } => message,
         });
+        builder.set_publicity(self.publicity(constructor.publicity));
         let type_builder = builder.reborrow().init_type();
         self.build_type(type_builder, &constructor.typ);
         self.build_types(
@@ -198,15 +211,23 @@ impl<'a> ModuleEncoder<'a> {
         mut builder: value_constructor::Builder<'_>,
         constructor: &ValueConstructor,
     ) {
-        builder.set_public(constructor.public);
         builder.set_deprecated(match &constructor.deprecation {
             Deprecation::NotDeprecated => "",
             Deprecation::Deprecated { message } => message,
         });
+        builder.set_publicity(self.publicity(constructor.publicity));
         self.build_type(builder.reborrow().init_type(), &constructor.type_);
         self.build_value_constructor_variant(builder.init_variant(), &constructor.variant);
     }
 
+    fn publicity(&self, publicity: Publicity) -> crate::schema_capnp::Publicity {
+        match publicity {
+            Publicity::Public => crate::schema_capnp::Publicity::Public,
+            Publicity::Private => crate::schema_capnp::Publicity::Private,
+            Publicity::Internal => crate::schema_capnp::Publicity::Internal,
+        }
+    }
+
     fn build_src_span(&mut self, mut builder: src_span::Builder<'_>, span: SrcSpan) {
         builder.set_start(span.start);
         builder.set_end(span.end);
@@ -500,7 +521,9 @@ impl<'a> ModuleEncoder<'a> {
         implementations: Implementations,
     ) {
         builder.set_gleam(implementations.gleam);
-        builder.set_erlang(implementations.uses_erlang_externals);
-        builder.set_javascript(implementations.uses_javascript_externals);
+        builder.set_uses_erlang_externals(implementations.uses_erlang_externals);
+        builder.set_uses_javascript_externals(implementations.uses_javascript_externals);
+        builder.set_can_run_on_erlang(implementations.can_run_on_erlang);
+        builder.set_can_run_on_javascript(implementations.can_run_on_javascript);
     }
 }
