@@ -4,10 +4,11 @@ use type_::{AccessorsMap, FieldMap, RecordAccessor};
 use super::*;
 use crate::{
     ast::{
-        BitArrayOption, BitArraySegment, CallArg, Constant, SrcSpan, TypedConstant,
+        BitArrayOption, BitArraySegment, CallArg, Constant, Publicity, SrcSpan, TypedConstant,
         TypedConstantBitArraySegmentOption,
     },
     build::Origin,
+    line_numbers::LineNumbers,
     type_::{
         self, expression::Implementations, Deprecation, ModuleInterface, Type, TypeConstructor,
         TypeValueConstructor, TypeValueConstructorField, TypeVariantConstructors, ValueConstructor,
@@ -29,6 +30,7 @@ fn roundtrip(input: &ModuleInterface) -> ModuleInterface {
 
 fn constant_module(constant: TypedConstant) -> ModuleInterface {
     ModuleInterface {
+        is_internal: true,
         contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
@@ -40,7 +42,7 @@ fn constant_module(constant: TypedConstant) -> ModuleInterface {
         values: [(
             "one".into(),
             ValueConstructor {
-                public: true,
+                publicity: Publicity::Public,
                 deprecation: Deprecation::NotDeprecated,
                 type_: type_::int(),
                 variant: ValueConstructorVariant::ModuleConstant {
@@ -52,11 +54,15 @@ fn constant_module(constant: TypedConstant) -> ModuleInterface {
                         gleam: true,
                         uses_erlang_externals: false,
                         uses_javascript_externals: false,
+                        can_run_on_erlang: true,
+                        can_run_on_javascript: true,
                     },
                 },
             },
         )]
         .into(),
+        line_numbers: LineNumbers::new(""),
+        src_path: "some_path".into(),
     }
 }
 
@@ -78,6 +84,7 @@ fn bit_array_segment_option_module(option: TypedConstantBitArraySegmentOption) -
 #[test]
 fn empty_module() {
     let module = ModuleInterface {
+        is_internal: true,
         contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
@@ -87,6 +94,31 @@ fn empty_module() {
         values: HashMap::new(),
         unused_imports: Vec::new(),
         accessors: HashMap::new(),
+        line_numbers: LineNumbers::new(""),
+        src_path: "some_path".into(),
+    };
+    assert_eq!(roundtrip(&module), module);
+}
+
+#[test]
+fn with_line_numbers() {
+    let module = ModuleInterface {
+        is_internal: false,
+        contains_todo: false,
+        package: "some_package".into(),
+        origin: Origin::Src,
+        name: "one/two".into(),
+        types: HashMap::new(),
+        types_value_constructors: HashMap::new(),
+        values: HashMap::new(),
+        unused_imports: Vec::new(),
+        accessors: HashMap::new(),
+        line_numbers: LineNumbers::new(
+            "const a = 1
+        const b = 2
+        const c = 3",
+        ),
+        src_path: "some_path".into(),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -94,6 +126,7 @@ fn empty_module() {
 #[test]
 fn module_with_private_type() {
     let module = ModuleInterface {
+        is_internal: false,
         contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
@@ -102,7 +135,7 @@ fn module_with_private_type() {
             "ListIntType".into(),
             TypeConstructor {
                 typ: type_::list(type_::int()),
-                public: false,
+                publicity: Publicity::Private,
                 origin: Default::default(),
                 module: "the/module".into(),
                 parameters: vec![],
@@ -114,6 +147,8 @@ fn module_with_private_type() {
         values: HashMap::new(),
         unused_imports: Vec::new(),
         accessors: HashMap::new(),
+        line_numbers: LineNumbers::new(""),
+        src_path: "some_path".into(),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -121,6 +156,7 @@ fn module_with_private_type() {
 #[test]
 fn module_with_unused_import() {
     let module = ModuleInterface {
+        is_internal: false,
         contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
@@ -133,6 +169,8 @@ fn module_with_unused_import() {
         ],
         accessors: HashMap::new(),
         values: HashMap::new(),
+        line_numbers: LineNumbers::new(""),
+        src_path: "some_path".into(),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -140,6 +178,7 @@ fn module_with_unused_import() {
 #[test]
 fn module_with_app_type() {
     let module = ModuleInterface {
+        is_internal: false,
         contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
@@ -148,7 +187,7 @@ fn module_with_app_type() {
             "ListIntType".into(),
             TypeConstructor {
                 typ: type_::list(type_::int()),
-                public: true,
+                publicity: Publicity::Public,
                 origin: Default::default(),
                 module: "the/module".into(),
                 parameters: vec![],
@@ -160,6 +199,8 @@ fn module_with_app_type() {
         values: HashMap::new(),
         unused_imports: Vec::new(),
         accessors: HashMap::new(),
+        line_numbers: LineNumbers::new(""),
+        src_path: "some_path".into(),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -167,6 +208,7 @@ fn module_with_app_type() {
 #[test]
 fn module_with_fn_type() {
     let module = ModuleInterface {
+        is_internal: false,
         contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
@@ -175,7 +217,7 @@ fn module_with_fn_type() {
             "FnType".into(),
             TypeConstructor {
                 typ: type_::fn_(vec![type_::nil(), type_::float()], type_::int()),
-                public: true,
+                publicity: Publicity::Public,
                 origin: Default::default(),
                 module: "the/module".into(),
                 parameters: vec![],
@@ -187,6 +229,8 @@ fn module_with_fn_type() {
         values: HashMap::new(),
         unused_imports: Vec::new(),
         accessors: HashMap::new(),
+        line_numbers: LineNumbers::new(""),
+        src_path: "some_path".into(),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -194,6 +238,7 @@ fn module_with_fn_type() {
 #[test]
 fn module_with_tuple_type() {
     let module = ModuleInterface {
+        is_internal: false,
         contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
@@ -202,7 +247,7 @@ fn module_with_tuple_type() {
             "TupleType".into(),
             TypeConstructor {
                 typ: type_::tuple(vec![type_::nil(), type_::float(), type_::int()]),
-                public: true,
+                publicity: Publicity::Public,
                 origin: Default::default(),
                 module: "the/module".into(),
                 parameters: vec![],
@@ -214,6 +259,8 @@ fn module_with_tuple_type() {
         values: HashMap::new(),
         unused_imports: Vec::new(),
         accessors: HashMap::new(),
+        line_numbers: LineNumbers::new(""),
+        src_path: "some_path".into(),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -227,6 +274,7 @@ fn module_with_generic_type() {
 
     fn make(t1: Arc<Type>, t2: Arc<Type>) -> ModuleInterface {
         ModuleInterface {
+            is_internal: false,
             contains_todo: false,
             package: "some_package".into(),
             origin: Origin::Src,
@@ -235,7 +283,7 @@ fn module_with_generic_type() {
                 "TupleType".into(),
                 TypeConstructor {
                     typ: type_::tuple(vec![t1.clone(), t1.clone(), t2.clone()]),
-                    public: true,
+                    publicity: Publicity::Public,
                     origin: Default::default(),
                     module: "the/module".into(),
                     parameters: vec![t1, t2],
@@ -247,6 +295,8 @@ fn module_with_generic_type() {
             values: HashMap::new(),
             unused_imports: Vec::new(),
             accessors: HashMap::new(),
+            line_numbers: LineNumbers::new(""),
+            src_path: "some_path".into(),
         }
     }
 
@@ -260,6 +310,7 @@ fn module_with_type_links() {
 
     fn make(type_: Arc<Type>) -> ModuleInterface {
         ModuleInterface {
+            is_internal: false,
             contains_todo: false,
             package: "some_package".into(),
             origin: Origin::Src,
@@ -268,7 +319,7 @@ fn module_with_type_links() {
                 "SomeType".into(),
                 TypeConstructor {
                     typ: type_,
-                    public: true,
+                    publicity: Publicity::Public,
                     origin: Default::default(),
                     module: "a".into(),
                     parameters: vec![],
@@ -280,6 +331,8 @@ fn module_with_type_links() {
             values: HashMap::new(),
             unused_imports: Vec::new(),
             accessors: HashMap::new(),
+            line_numbers: LineNumbers::new(""),
+            src_path: "some_path".into(),
         }
     }
 
@@ -289,6 +342,7 @@ fn module_with_type_links() {
 #[test]
 fn module_type_to_constructors_mapping() {
     let module = ModuleInterface {
+        is_internal: false,
         contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
@@ -308,6 +362,8 @@ fn module_type_to_constructors_mapping() {
         unused_imports: Default::default(),
         accessors: HashMap::new(),
         values: HashMap::new(),
+        line_numbers: LineNumbers::new(""),
+        src_path: "some_path".into(),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -316,6 +372,7 @@ fn module_type_to_constructors_mapping() {
 #[test]
 fn module_fn_value() {
     let module = ModuleInterface {
+        is_internal: false,
         contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
@@ -327,7 +384,7 @@ fn module_fn_value() {
         values: [(
             "one".into(),
             ValueConstructor {
-                public: true,
+                publicity: Publicity::Public,
                 deprecation: Deprecation::NotDeprecated,
                 type_: type_::int(),
                 variant: ValueConstructorVariant::ModuleFn {
@@ -344,11 +401,15 @@ fn module_fn_value() {
                         gleam: true,
                         uses_erlang_externals: false,
                         uses_javascript_externals: false,
+                        can_run_on_erlang: true,
+                        can_run_on_javascript: true,
                     },
                 },
             },
         )]
         .into(),
+        line_numbers: LineNumbers::new(""),
+        src_path: "some_path".into(),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -356,6 +417,7 @@ fn module_fn_value() {
 #[test]
 fn deprecated_module_fn_value() {
     let module = ModuleInterface {
+        is_internal: false,
         contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
@@ -367,7 +429,7 @@ fn deprecated_module_fn_value() {
         values: [(
             "one".into(),
             ValueConstructor {
-                public: true,
+                publicity: Publicity::Public,
                 deprecation: Deprecation::Deprecated {
                     message: "wibble wobble".into(),
                 },
@@ -386,11 +448,15 @@ fn deprecated_module_fn_value() {
                         gleam: true,
                         uses_erlang_externals: false,
                         uses_javascript_externals: false,
+                        can_run_on_erlang: true,
+                        can_run_on_javascript: true,
                     },
                 },
             },
         )]
         .into(),
+        line_numbers: LineNumbers::new(""),
+        src_path: "some_path".into(),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -398,6 +464,7 @@ fn deprecated_module_fn_value() {
 #[test]
 fn private_module_fn_value() {
     let module = ModuleInterface {
+        is_internal: false,
         contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
@@ -409,7 +476,7 @@ fn private_module_fn_value() {
         values: [(
             "one".into(),
             ValueConstructor {
-                public: false,
+                publicity: Publicity::Private,
                 deprecation: Deprecation::NotDeprecated,
                 type_: type_::int(),
                 variant: ValueConstructorVariant::ModuleFn {
@@ -426,11 +493,15 @@ fn private_module_fn_value() {
                         gleam: true,
                         uses_erlang_externals: false,
                         uses_javascript_externals: false,
+                        can_run_on_erlang: true,
+                        can_run_on_javascript: true,
                     },
                 },
             },
         )]
         .into(),
+        line_numbers: LineNumbers::new(""),
+        src_path: "some_path".into(),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -440,6 +511,7 @@ fn private_module_fn_value() {
 #[test]
 fn module_fn_value_regression() {
     let module = ModuleInterface {
+        is_internal: false,
         contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
@@ -451,7 +523,7 @@ fn module_fn_value_regression() {
         values: [(
             "one".into(),
             ValueConstructor {
-                public: true,
+                publicity: Publicity::Public,
                 deprecation: Deprecation::NotDeprecated,
                 type_: type_::int(),
                 variant: ValueConstructorVariant::ModuleFn {
@@ -468,11 +540,15 @@ fn module_fn_value_regression() {
                         gleam: true,
                         uses_erlang_externals: false,
                         uses_javascript_externals: false,
+                        can_run_on_erlang: true,
+                        can_run_on_javascript: true,
                     },
                 },
             },
         )]
         .into(),
+        line_numbers: LineNumbers::new(""),
+        src_path: "some_path".into(),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -481,6 +557,7 @@ fn module_fn_value_regression() {
 #[test]
 fn module_fn_value_with_field_map() {
     let module = ModuleInterface {
+        is_internal: false,
         contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
@@ -492,7 +569,7 @@ fn module_fn_value_with_field_map() {
         values: [(
             "one".into(),
             ValueConstructor {
-                public: true,
+                publicity: Publicity::Public,
                 deprecation: Deprecation::NotDeprecated,
                 type_: type_::int(),
                 variant: ValueConstructorVariant::ModuleFn {
@@ -509,11 +586,15 @@ fn module_fn_value_with_field_map() {
                         gleam: true,
                         uses_erlang_externals: false,
                         uses_javascript_externals: false,
+                        can_run_on_erlang: true,
+                        can_run_on_javascript: true,
                     },
                 },
             },
         )]
         .into(),
+        line_numbers: LineNumbers::new(""),
+        src_path: "some_path".into(),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -524,6 +605,7 @@ fn record_value() {
     let mut random = rand::thread_rng();
 
     let module = ModuleInterface {
+        is_internal: false,
         contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
@@ -535,7 +617,7 @@ fn record_value() {
         values: [(
             "one".into(),
             ValueConstructor {
-                public: true,
+                publicity: Publicity::Public,
                 deprecation: Deprecation::NotDeprecated,
                 type_: type_::int(),
                 variant: ValueConstructorVariant::Record {
@@ -554,6 +636,8 @@ fn record_value() {
             },
         )]
         .into(),
+        line_numbers: LineNumbers::new(""),
+        src_path: "some_path".into(),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -564,6 +648,7 @@ fn record_value_with_field_map() {
     let mut random = rand::thread_rng();
 
     let module = ModuleInterface {
+        is_internal: false,
         contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
@@ -575,7 +660,7 @@ fn record_value_with_field_map() {
         values: [(
             "one".into(),
             ValueConstructor {
-                public: true,
+                publicity: Publicity::Public,
                 deprecation: Deprecation::NotDeprecated,
                 type_: type_::int(),
                 variant: ValueConstructorVariant::Record {
@@ -597,6 +682,8 @@ fn record_value_with_field_map() {
             },
         )]
         .into(),
+        line_numbers: LineNumbers::new(""),
+        src_path: "some_path".into(),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -605,6 +692,7 @@ fn record_value_with_field_map() {
 #[test]
 fn accessors() {
     let module = ModuleInterface {
+        is_internal: false,
         contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
@@ -617,7 +705,7 @@ fn accessors() {
             (
                 "one".into(),
                 AccessorsMap {
-                    public: true,
+                    publicity: Publicity::Public,
                     type_: type_::int(),
                     accessors: [
                         (
@@ -643,7 +731,7 @@ fn accessors() {
             (
                 "two".into(),
                 AccessorsMap {
-                    public: true,
+                    publicity: Publicity::Public,
                     type_: type_::int(),
                     accessors: [(
                         "a".into(),
@@ -658,6 +746,8 @@ fn accessors() {
             ),
         ]
         .into(),
+        line_numbers: LineNumbers::new(""),
+        src_path: "some_path".into(),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -796,7 +886,7 @@ fn constant_var() {
         name: "one_original".into(),
         typ: type_::int(),
         constructor: Some(Box::from(ValueConstructor {
-            public: true,
+            publicity: Publicity::Public,
             deprecation: Deprecation::NotDeprecated,
             type_: type_::int(),
             variant: ValueConstructorVariant::ModuleConstant {
@@ -808,12 +898,15 @@ fn constant_var() {
                     gleam: true,
                     uses_erlang_externals: false,
                     uses_javascript_externals: false,
+                    can_run_on_erlang: true,
+                    can_run_on_javascript: true,
                 },
             },
         })),
     };
 
     let module = ModuleInterface {
+        is_internal: false,
         contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
@@ -826,7 +919,7 @@ fn constant_var() {
             (
                 "one".into(),
                 ValueConstructor {
-                    public: true,
+                    publicity: Publicity::Public,
                     deprecation: Deprecation::NotDeprecated,
                     type_: type_::int(),
                     variant: ValueConstructorVariant::ModuleConstant {
@@ -838,6 +931,8 @@ fn constant_var() {
                             gleam: true,
                             uses_erlang_externals: false,
                             uses_javascript_externals: false,
+                            can_run_on_erlang: true,
+                            can_run_on_javascript: true,
                         },
                     },
                 },
@@ -845,7 +940,7 @@ fn constant_var() {
             (
                 "one_original".into(),
                 ValueConstructor {
-                    public: true,
+                    publicity: Publicity::Public,
                     deprecation: Deprecation::NotDeprecated,
                     type_: type_::int(),
                     variant: ValueConstructorVariant::ModuleConstant {
@@ -857,12 +952,16 @@ fn constant_var() {
                             gleam: true,
                             uses_erlang_externals: false,
                             uses_javascript_externals: false,
+                            can_run_on_erlang: true,
+                            can_run_on_javascript: true,
                         },
                     },
                 },
             ),
         ]
         .into(),
+        line_numbers: LineNumbers::new(""),
+        src_path: "some_path".into(),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -1027,6 +1126,7 @@ fn constant_bit_array_native() {
 #[test]
 fn deprecated_type() {
     let module = ModuleInterface {
+        is_internal: false,
         contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
@@ -1035,7 +1135,7 @@ fn deprecated_type() {
             "ListIntType".into(),
             TypeConstructor {
                 typ: type_::list(type_::int()),
-                public: true,
+                publicity: Publicity::Public,
                 origin: Default::default(),
                 module: "the/module".into(),
                 parameters: vec![],
@@ -1049,6 +1149,8 @@ fn deprecated_type() {
         values: HashMap::new(),
         unused_imports: Vec::new(),
         accessors: HashMap::new(),
+        line_numbers: LineNumbers::new(""),
+        src_path: "some_path".into(),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -1058,6 +1160,7 @@ fn contains_todo() {
     let module = ModuleInterface {
         contains_todo: true,
         //             ^^^^ It does, it does!
+        is_internal: false,
         package: "some_package".into(),
         origin: Origin::Src,
         name: "a/b".into(),
@@ -1066,6 +1169,8 @@ fn contains_todo() {
         values: HashMap::new(),
         unused_imports: Vec::new(),
         accessors: HashMap::new(),
+        line_numbers: LineNumbers::new(""),
+        src_path: "some_path".into(),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -1073,6 +1178,7 @@ fn contains_todo() {
 #[test]
 fn module_fn_value_with_external_implementations() {
     let module = ModuleInterface {
+        is_internal: false,
         contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
@@ -1084,7 +1190,7 @@ fn module_fn_value_with_external_implementations() {
         values: [(
             "one".into(),
             ValueConstructor {
-                public: true,
+                publicity: Publicity::Public,
                 deprecation: Deprecation::NotDeprecated,
                 type_: type_::int(),
                 variant: ValueConstructorVariant::ModuleFn {
@@ -1101,11 +1207,61 @@ fn module_fn_value_with_external_implementations() {
                         gleam: false,
                         uses_erlang_externals: true,
                         uses_javascript_externals: true,
+                        can_run_on_erlang: false,
+                        can_run_on_javascript: true,
                     },
                 },
             },
         )]
         .into(),
+        line_numbers: LineNumbers::new(""),
+        src_path: "some_path".into(),
+    };
+
+    assert_eq!(roundtrip(&module), module);
+}
+
+#[test]
+fn internal_module_fn() {
+    let module = ModuleInterface {
+        is_internal: false,
+        contains_todo: false,
+        package: "some_package".into(),
+        origin: Origin::Src,
+        name: "a/b/c".into(),
+        types: HashMap::new(),
+        types_value_constructors: HashMap::new(),
+        unused_imports: Vec::new(),
+        accessors: HashMap::new(),
+        values: [(
+            "one".into(),
+            ValueConstructor {
+                publicity: Publicity::Internal,
+                deprecation: Deprecation::NotDeprecated,
+                type_: type_::int(),
+                variant: ValueConstructorVariant::ModuleFn {
+                    documentation: Some("wabble!".into()),
+                    name: "one".into(),
+                    field_map: None,
+                    module: "a".into(),
+                    arity: 5,
+                    location: SrcSpan {
+                        start: 52,
+                        end: 1100,
+                    },
+                    implementations: Implementations {
+                        gleam: false,
+                        uses_erlang_externals: true,
+                        uses_javascript_externals: true,
+                        can_run_on_erlang: true,
+                        can_run_on_javascript: true,
+                    },
+                },
+            },
+        )]
+        .into(),
+        line_numbers: LineNumbers::new(""),
+        src_path: "some_path".into(),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -1115,6 +1271,7 @@ fn module_fn_value_with_external_implementations() {
 #[test]
 fn type_variable_ids_in_constructors_are_shared() {
     let module = ModuleInterface {
+        is_internal: false,
         contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
@@ -1143,6 +1300,8 @@ fn type_variable_ids_in_constructors_are_shared() {
         unused_imports: Vec::new(),
         accessors: HashMap::new(),
         values: [].into(),
+        line_numbers: LineNumbers::new(""),
+        src_path: "some_path".into(),
     };
 
     let expected = HashMap::from([(
