@@ -8,6 +8,7 @@ use crate::{
     analyse::TargetSupport,
     build::{Module, Origin, Package, Target},
     config::{Docs, ErlangConfig, JavaScriptConfig, PackageConfig, Repository},
+    line_numbers::LineNumbers,
     type_::PRELUDE_MODULE_NAME,
     uid::UniqueIdGenerator,
     warning::TypeWarningEmitter,
@@ -70,16 +71,21 @@ pub fn compile_package(
         let parsed = crate::parse::parse_module(dep_src).expect("dep syntax error");
         let mut ast = parsed.module;
         ast.name = dep_name.into();
+        let line_numbers = LineNumbers::new(dep_src);
+        let mut config = PackageConfig::default();
+        config.name = dep_package.into();
         let dep = crate::analyse::infer_module::<()>(
             Target::Erlang,
             &ids,
             ast,
             Origin::Src,
-            &dep_package.into(),
             &modules,
             &TypeWarningEmitter::null(),
             &std::collections::HashMap::new(),
             TargetSupport::Enforced,
+            line_numbers,
+            &config,
+            "".into(),
         )
         .expect("should successfully infer");
         let _ = modules.insert(dep_name.into(), dep.type_info);
@@ -93,16 +99,20 @@ pub fn compile_package(
         .unwrap_or("my/module".into());
 
     ast.name = module_name.clone();
+    let mut config = PackageConfig::default();
+    config.name = "my_package".into();
     let ast = crate::analyse::infer_module::<()>(
         Target::Erlang,
         &ids,
         ast,
         Origin::Src,
-        &"my_package".into(),
         &modules,
         &TypeWarningEmitter::null(),
         &direct_dependencies,
         TargetSupport::Enforced,
+        LineNumbers::new(src),
+        &config,
+        "".into(),
     )
     .expect("should successfully infer");
 
@@ -180,6 +190,18 @@ type Wob = Int
     );
 }
 
+#[test]
+pub fn internal_definitions_are_not_included() {
+    assert_package_interface!(
+        "
+@internal pub const float = 1.1
+@internal pub fn main() {}
+@internal pub type Foo
+@internal pub type Bar = Int
+"
+    );
+}
+
 #[test]
 pub fn opaque_constructors_are_not_exposed() {
     assert_package_interface!("pub opaque type Wibble { Wob }")
