@@ -611,6 +611,19 @@ pub fn main() -> Nil {
     );
 }
 
+#[test]
+fn multiple_internal_attributes() {
+    assert_module_error!(
+        r#"
+@internal
+@internal
+pub fn main() -> Nil {
+  Nil
+}
+"#
+    );
+}
+
 #[test]
 fn attributes_with_no_definition() {
     assert_module_error!(
@@ -700,3 +713,310 @@ fn add(x, y) {
 "#
     );
 }
+
+#[test]
+fn list_spread_with_no_tail_in_the_middle_of_a_list() {
+    assert_module_error!(
+        r#"
+pub fn main() -> Nil {
+  let xs = [1, 2, 3]
+  [1, 2, .., 3 + 3, 4]
+}
+"#
+    );
+}
+
+#[test]
+fn list_spread_followed_by_extra_items() {
+    assert_module_error!(
+        r#"
+pub fn main() -> Nil {
+  let xs = [1, 2, 3]
+  [1, 2, ..xs, 3 + 3, 4]
+}
+"#
+    );
+}
+
+// Tests for nested tuples and structs in tuples
+// https://github.com/gleam-lang/gleam/issues/1980
+
+#[test]
+fn nested_tuples() {
+    assert_parse!(
+        r#"
+let tup = #(#(5, 6))
+{tup.0}.1
+"#
+    );
+}
+
+#[test]
+fn nested_tuples_no_block() {
+    assert_parse!(
+        r#"
+let tup = #(#(5, 6))
+tup.0.1
+"#
+    );
+}
+
+#[test]
+fn deeply_nested_tuples() {
+    assert_parse!(
+        r#"
+let tup = #(#(#(#(4))))
+{{{tup.0}.0}.0}.0
+"#
+    );
+}
+
+#[test]
+fn deeply_nested_tuples_no_block() {
+    assert_parse!(
+        r#"
+let tup = #(#(#(#(4))))
+tup.0.0.0.0
+"#
+    );
+}
+
+#[test]
+fn inner_single_quote_parses() {
+    assert_parse!(
+        r#"
+let a = "inner 'quotes'"
+"#
+    );
+}
+
+#[test]
+fn string_single_char_suggestion() {
+    assert_module_error!(
+        "
+    pub fn main() {
+        let a = 'example'
+      }
+    "
+    );
+}
+
+#[test]
+fn private_internal_const() {
+    assert_module_error!(
+        "
+@internal
+const wibble = 1
+"
+    );
+}
+
+#[test]
+fn private_internal_type_alias() {
+    assert_module_error!(
+        "
+@internal
+type Alias = Int
+"
+    );
+}
+
+#[test]
+fn private_internal_function() {
+    assert_module_error!(
+        "
+@internal
+fn wibble() { todo }
+"
+    );
+}
+
+#[test]
+fn private_internal_type() {
+    assert_module_error!(
+        "
+@internal
+type Wibble {
+  Wibble
+}
+"
+    );
+}
+
+#[test]
+fn wrong_record_access_pattern() {
+    assert_module_error!(
+        "
+pub fn main() {
+  case wibble {
+    wibble.thing -> 1
+  }
+}
+"
+    );
+}
+
+#[test]
+fn tuple_invalid_expr() {
+    assert_module_error!(
+        "
+fn main() {
+    #(1, 2, const)
+}
+"
+    );
+}
+
+#[test]
+fn bit_array_invalid_segment() {
+    assert_module_error!(
+        "
+fn main() {
+    <<72, 101, 108, 108, 111, 44, 32, 74, 111, 101, const>>
+}
+"
+    );
+}
+
+#[test]
+fn case_invalid_expression() {
+    assert_module_error!(
+        "
+fn main() {
+    case 1, type {
+        _, _ -> 0
+    }
+}
+"
+    );
+}
+
+#[test]
+fn case_invalid_case_pattern() {
+    assert_module_error!(
+        "
+fn main() {
+    case 1 {
+        -> -> 0
+    }
+}
+"
+    );
+}
+
+#[test]
+fn use_invalid_assignments() {
+    assert_module_error!(
+        "
+fn main() {
+    use fn <- result.try(get_username())
+}
+"
+    );
+}
+
+#[test]
+fn assignment_pattern_invalid_tuple() {
+    assert_module_error!(
+        "
+fn main() {
+    let #(a, case, c) = #(1, 2, 3)
+}
+"
+    );
+}
+
+#[test]
+fn assignment_pattern_invalid_bit_segment() {
+    assert_module_error!(
+        "
+fn main() {
+    let <<b1, pub>> = <<24, 3>>
+}
+"
+    );
+}
+
+#[test]
+fn type_invalid_constructor() {
+    assert_module_error!(
+        "
+type A { 
+    A(String)
+    type
+}
+"
+    );
+}
+
+#[test]
+fn type_invalid_type_name() {
+    assert_module_error!(
+        "
+type A(a, type) { 
+    A
+}
+"
+    );
+}
+
+#[test]
+fn type_invalid_constructor_arg() {
+    assert_module_error!(
+        "
+type A { 
+    A(type: String)
+}
+"
+    );
+}
+
+#[test]
+fn function_type_invalid_param_type() {
+    assert_module_error!(
+        "
+fn f(g: fn(Int, 1) -> Int) -> Int {
+  g(0, 1)
+}
+"
+    );
+}
+
+#[test]
+fn const_invalid_tuple() {
+    assert_module_error!(
+        "
+const a = #(1, 2, <-)
+"
+    );
+}
+
+#[test]
+fn const_invalid_list() {
+    assert_module_error!(
+        "
+const a = [1, 2, <-]
+"
+    );
+}
+
+#[test]
+fn const_invalid_bit_array_segment() {
+    assert_module_error!(
+        "
+const a = <<1, 2, <->>
+"
+    );
+}
+
+#[test]
+fn const_invalid_record_constructor() {
+    assert_module_error!(
+        "
+type A {
+    A(String, Int)
+}
+const a = A(\"a\", let)
+"
+    );
+}
