@@ -10,6 +10,7 @@ pub mod pretty;
 #[cfg(test)]
 pub mod tests;
 
+use camino::Utf8PathBuf;
 use ecow::EcoString;
 pub use environment::*;
 pub use error::{Error, UnifyErrorSituation, Warning};
@@ -20,12 +21,14 @@ use serde::Serialize;
 
 use crate::{
     ast::{
-        ArgNames, BitArraySegment, CallArg, Constant, DefinitionLocation, Pattern, SrcSpan,
-        TypedConstant, TypedExpr, TypedPattern, TypedPatternBitArraySegment, TypedRecordUpdateArg,
-        UntypedMultiPattern, UntypedPattern, UntypedRecordUpdateArg,
+        ArgNames, BitArraySegment, CallArg, Constant, DefinitionLocation, Pattern, Publicity,
+        SrcSpan, TypedConstant, TypedExpr, TypedPattern, TypedPatternBitArraySegment,
+        TypedRecordUpdateArg, UntypedMultiPattern, UntypedPattern, UntypedRecordUpdateArg,
     },
     bit_array,
     build::{Origin, Target},
+    line_numbers::LineNumbers,
+    type_::expression::Implementations,
 };
 use error::*;
 use hydrator::Hydrator;
@@ -37,8 +40,6 @@ use std::{
     sync::Arc,
 };
 
-use self::expression::Implementations;
-
 pub trait HasType {
     fn type_(&self) -> Arc<Type>;
 }
@@ -54,7 +55,7 @@ pub enum Type {
     /// that defines the type.
     ///
     Named {
-        public: bool,
+        publicity: Publicity,
         package: EcoString,
         module: EcoString,
         name: EcoString,
@@ -83,25 +84,44 @@ impl Type {
     pub fn is_result_constructor(&self) -> bool {
         match self {
             Type::Fn { retrn, .. } => retrn.is_result(),
+            Type::Var { type_ } => type_.borrow().is_result(),
             _ => false,
         }
     }
 
     pub fn is_result(&self) -> bool {
-        matches!(self, Self::Named { name, module, .. } if "Result" == name && is_prelude_module(module))
+        match self {
+            Self::Named { name, module, .. } => "Result" == name && is_prelude_module(module),
+            Self::Var { type_ } => type_.borrow().is_result(),
+            _ => false,
+        }
     }
 
     pub fn is_unbound(&self) -> bool {
-        matches!(self, Self::Var { type_: typ } if typ.borrow().is_unbound())
+        match self {
+            Self::Var { type_: typ } => typ.borrow().is_unbound(),
+            _ => false,
+        }
     }
 
     pub fn is_variable(&self) -> bool {
-        matches!(self, Self::Var { type_: typ } if typ.borrow().is_variable())
+        match self {
+            Self::Var { type_: typ } => typ.borrow().is_variable(),
+            _ => false,
+        }
+    }
+
+    pub fn is_type_variable(&self) -> bool {
+        match self {
+            Self::Var { type_: typ } => typ.borrow().is_variable(),
+            _ => false,
+        }
     }
 
     pub fn return_type(&self) -> Option<Arc<Self>> {
         match self {
             Self::Fn { retrn, .. } => Some(retrn.clone()),
+            Type::Var { type_ } => type_.borrow().return_type(),
             _ => None,
         }
     }
@@ -109,6 +129,7 @@ impl Type {
     pub fn fn_types(&self) -> Option<(Vec<Arc<Self>>, Arc<Self>)> {
         match self {
             Self::Fn { args, retrn, .. } => Some((args.clone(), retrn.clone())),
+            Self::Var { type_ } => type_.borrow().fn_types(),
             _ => None,
         }
     }
@@ -183,7 +204,7 @@ impl Type {
     // TODO: specialise this to just List.
     pub fn get_app_args(
         &self,
-        public: bool,
+        publicity: Publicity,
         package: &str,
         module: &str,
         name: &str,
@@ -207,7 +228,14 @@ impl Type {
             Self::Var { type_: typ } => {
                 let args: Vec<_> = match typ.borrow().deref() {
                     TypeVar::Link { type_: typ } => {
-                        return typ.get_app_args(public, package, module, name, arity, environment);
+                        return typ.get_app_args(
+                            publicity,
+                            package,
+                            module,
+                            name,
+                            arity,
+                            environment,
+                        );
                     }
 
                     TypeVar::Unbound { .. } => {
@@ -225,7 +253,7 @@ impl Type {
                         package: package.into(),
                         module: module.into(),
                         args: args.clone(),
-                        public,
+                        publicity,
                     }),
                 };
                 Some(args)
@@ -237,7 +265,10 @@ impl Type {
 
     pub fn find_private_type(&self) -> Option<Self> {
         match self {
-            Self::Named { public: false, .. } => Some(self.clone()),
+            Self::Named {
+                publicity: Publicity::Private,
+                ..
+            } => Some(self.clone()),
 
             Self::Named { args, .. } => args.iter().find_map(|t| t.find_private_type()),
 
@@ -257,6 +288,25 @@ impl Type {
         }
     }
 
+    pub fn find_internal_type(&self) -> Option<Self> {
+        match self {
+            Self::Named { publicity, .. } if publicity.is_internal() => Some(self.clone()),
+
+            Self::Named { args, .. } => args.iter().find_map(|t| t.find_internal_type()),
+
+            Self::Tuple { elems, .. } => elems.iter().find_map(|t| t.find_internal_type()),
+
+            Self::Fn { retrn, args, .. } => retrn
+                .find_internal_type()
+                .or_else(|| args.iter().find_map(|t| t.find_internal_type())),
+
+            Self::Var { type_: typ, .. } => match typ.borrow().deref() {
+                TypeVar::Unbound { .. } | TypeVar::Generic { .. } => None,
+                TypeVar::Link { type_: typ, .. } => typ.find_internal_type(),
+            },
+        }
+    }
+
     pub fn fn_arity(&self) -> Option<usize> {
         match self {
             Self::Fn { args, .. } => Some(args.len()),
@@ -276,7 +326,7 @@ pub fn collapse_links(t: Arc<Type>) -> Arc<Type> {
 
 #[derive(Debug, Clone, PartialEq, Eq)]
 pub struct AccessorsMap {
-    pub public: bool,
+    pub publicity: Publicity,
     pub type_: Arc<Type>,
     pub accessors: HashMap<EcoString, RecordAccessor>,
 }
@@ -419,12 +469,21 @@ impl ValueConstructorVariant {
         matches!(self, Self::ModuleFn { .. })
     }
 
+    pub fn is_record(&self) -> bool {
+        match self {
+            Self::Record { .. } => true,
+            _ => false,
+        }
+    }
+
     pub fn implementations(&self) -> Implementations {
         match self {
             ValueConstructorVariant::Record { .. }
             | ValueConstructorVariant::LocalConstant { .. }
             | ValueConstructorVariant::LocalVariable { .. } => Implementations {
                 gleam: true,
+                can_run_on_erlang: true,
+                can_run_on_javascript: true,
                 uses_javascript_externals: false,
                 uses_erlang_externals: false,
             },
@@ -511,6 +570,14 @@ pub struct ModuleInterface {
     pub accessors: HashMap<EcoString, AccessorsMap>,
     pub unused_imports: Vec<SrcSpan>,
     pub contains_todo: bool,
+    /// Used for mapping to original source locations on disk
+    pub line_numbers: LineNumbers,
+    /// Used for determining the source path of the module on disk
+    pub src_path: Utf8PathBuf,
+    // Whether the module is internal or not. Internal modules are technically
+    // importable by other packages but to do so is violating the contract of
+    // the package and as such is not recommended.
+    pub is_internal: bool,
 }
 
 /// Information on the constructors of a custom type.
@@ -576,7 +643,13 @@ pub struct TypeValueConstructorField {
 }
 
 impl ModuleInterface {
-    pub fn new(name: EcoString, origin: Origin, package: EcoString) -> Self {
+    pub fn new(
+        name: EcoString,
+        origin: Origin,
+        package: EcoString,
+        line_numbers: LineNumbers,
+        src_path: Utf8PathBuf,
+    ) -> Self {
         Self {
             name,
             origin,
@@ -587,12 +660,15 @@ impl ModuleInterface {
             accessors: Default::default(),
             unused_imports: Default::default(),
             contains_todo: false,
+            is_internal: false,
+            line_numbers,
+            src_path,
         }
     }
 
     pub fn get_public_value(&self, name: &str) -> Option<&ValueConstructor> {
         let value = self.values.get(name)?;
-        if value.public {
+        if value.publicity.is_importable() {
             Some(value)
         } else {
             None
@@ -601,7 +677,7 @@ impl ModuleInterface {
 
     pub fn get_public_type(&self, name: &str) -> Option<&TypeConstructor> {
         let type_ = self.types.get(name)?;
-        if type_.public {
+        if type_.publicity.is_importable() {
             Some(type_)
         } else {
             None
@@ -629,7 +705,7 @@ impl ModuleInterface {
     pub fn public_value_names(&self) -> Vec<EcoString> {
         self.values
             .iter()
-            .filter(|(_, v)| v.public)
+            .filter(|(_, v)| v.publicity.is_importable())
             .map(|(k, _)| k)
             .cloned()
             .collect_vec()
@@ -638,7 +714,7 @@ impl ModuleInterface {
     pub fn public_type_names(&self) -> Vec<EcoString> {
         self.types
             .iter()
-            .filter(|(_, v)| v.public)
+            .filter(|(_, v)| v.publicity.is_importable())
             .map(|(k, _)| k)
             .cloned()
             .collect_vec()
@@ -697,59 +773,86 @@ pub enum TypeVar {
 
 impl TypeVar {
     pub fn is_unbound(&self) -> bool {
-        matches!(self, Self::Unbound { .. })
+        match self {
+            Self::Unbound { .. } => true,
+            Self::Link { .. } | Self::Generic { .. } => false,
+        }
     }
 
     pub fn is_variable(&self) -> bool {
-        matches!(self, Self::Unbound { .. } | Self::Generic { .. })
+        match self {
+            Self::Unbound { .. } | Self::Generic { .. } => true,
+            Self::Link { type_ } => type_.is_type_variable(),
+        }
+    }
+
+    pub fn return_type(&self) -> Option<Arc<Type>> {
+        match self {
+            Self::Link { type_ } => type_.return_type(),
+            Self::Unbound { .. } | Self::Generic { .. } => None,
+        }
+    }
+
+    pub fn is_result(&self) -> bool {
+        match self {
+            Self::Link { type_ } => type_.is_result(),
+            Self::Unbound { .. } | Self::Generic { .. } => false,
+        }
+    }
+
+    pub fn fn_types(&self) -> Option<(Vec<Arc<Type>>, Arc<Type>)> {
+        match self {
+            Self::Link { type_ } => type_.fn_types(),
+            Self::Unbound { .. } | Self::Generic { .. } => None,
+        }
     }
 
     pub fn is_nil(&self) -> bool {
         match self {
             Self::Link { type_ } => type_.is_nil(),
-            _ => false,
+            Self::Unbound { .. } | Self::Generic { .. } => false,
         }
     }
 
     pub fn is_bool(&self) -> bool {
         match self {
             Self::Link { type_ } => type_.is_bool(),
-            _ => false,
+            Self::Unbound { .. } | Self::Generic { .. } => false,
         }
     }
 
     pub fn is_int(&self) -> bool {
         match self {
             Self::Link { type_ } => type_.is_int(),
-            _ => false,
+            Self::Unbound { .. } | Self::Generic { .. } => false,
         }
     }
 
     pub fn is_float(&self) -> bool {
         match self {
             Self::Link { type_ } => type_.is_float(),
-            _ => false,
+            Self::Unbound { .. } | Self::Generic { .. } => false,
         }
     }
 
     pub fn is_string(&self) -> bool {
         match self {
             Self::Link { type_ } => type_.is_string(),
-            _ => false,
+            Self::Unbound { .. } | Self::Generic { .. } => false,
         }
     }
 
     pub fn named_type_name(&self) -> Option<(EcoString, EcoString)> {
         match self {
             Self::Link { type_ } => type_.named_type_name(),
-            _ => None,
+            Self::Unbound { .. } | Self::Generic { .. } => None,
         }
     }
 }
 
 #[derive(Debug, Clone, PartialEq, Eq)]
 pub struct TypeConstructor {
-    pub public: bool,
+    pub publicity: Publicity,
     pub origin: SrcSpan,
     pub module: EcoString,
     pub parameters: Vec<Arc<Type>>,
@@ -765,7 +868,7 @@ impl TypeConstructor {
 
 #[derive(Debug, Clone, PartialEq, Eq)]
 pub struct ValueConstructor {
-    pub public: bool,
+    pub publicity: Publicity,
     pub deprecation: Deprecation,
     pub variant: ValueConstructorVariant,
     pub type_: Arc<Type>,
@@ -839,7 +942,7 @@ impl ValueConstructor {
 
 #[derive(Debug, Clone, PartialEq, Eq)]
 pub struct TypeAliasConstructor {
-    pub public: bool,
+    pub publicity: Publicity,
     pub module: EcoString,
     pub type_: Type,
     pub arity: usize,
@@ -974,7 +1077,7 @@ pub fn generalise(t: Arc<Type>) -> Arc<Type> {
         },
 
         Type::Named {
-            public,
+            publicity,
             module,
             package,
             name,
@@ -982,7 +1085,7 @@ pub fn generalise(t: Arc<Type>) -> Arc<Type> {
         } => {
             let args = args.iter().map(|t| generalise(t.clone())).collect();
             Arc::new(Type::Named {
-                public: *public,
+                publicity: *publicity,
                 module: module.clone(),
                 package: package.clone(),
                 name: name.clone(),
