@@ -20,6 +20,12 @@ pub struct UnknownType {
     pub name: EcoString,
 }
 
+#[derive(Debug, Eq, PartialEq, Clone, Copy)]
+pub enum RecordVariants {
+    HasVariants,
+    NoVariants,
+}
+
 #[derive(Debug, Eq, PartialEq, Clone)]
 pub enum Error {
     SrcImportingTest {
@@ -43,6 +49,7 @@ pub enum Error {
         location: SrcSpan,
         name: EcoString,
         variables: Vec<EcoString>,
+        type_with_name_in_scope: bool,
     },
 
     UnknownType {
@@ -90,6 +97,7 @@ pub enum Error {
         label: EcoString,
         fields: Vec<EcoString>,
         usage: FieldAccessUsage,
+        variants: RecordVariants,
     },
 
     IncorrectArity {
@@ -398,6 +406,10 @@ pub enum Warning {
         location: SrcSpan,
     },
 
+    UnusedValue {
+        location: SrcSpan,
+    },
+
     NoFieldsRecordUpdate {
         location: SrcSpan,
     },
@@ -477,6 +489,54 @@ pub enum Warning {
     UnreachableCaseClause {
         location: SrcSpan,
     },
+
+    /// This happens when someone tries to write a case expression where one of
+    /// the subjects is a literal tuple, for example:
+    ///
+    /// ```gleam
+    /// case #(wibble, wobble) { ... }
+    /// ```
+    ///
+    /// The tuple is redundant since we could do this:
+    ///
+    /// ```gleam
+    /// case wibble, wobble { ... }
+    /// ```
+    ///
+    CaseMatchOnLiteralTuple {
+        location: SrcSpan,
+    },
+
+    /// This happens when someone defines an external type (with no
+    /// constructors) and marks it as opqaue:
+    ///
+    /// ```gleam
+    /// opaque type External
+    /// ```
+    ///
+    /// Since an external type already has no constructors, marking it as
+    /// opaque is redundant.
+    ///
+    OpaqueExternalType {
+        location: SrcSpan,
+    },
+
+    /// This happens when an internal type is accidentally exposed in the public
+    /// API. Since internal types are excluded from documentation, completions
+    /// and the package interface, this would lead to poor developer experience.
+    ///
+    /// ```gleam
+    /// @internal type Wibble
+    ///
+    /// pub fn wibble(thing: Wibble) { todo }
+    /// //            ^^^^^^^^^^^^^ There would be no documentation
+    /// //                          explaining what `Wibble` is in the
+    /// //                          package's doc site.
+    /// ```
+    InternalTypeLeak {
+        location: SrcSpan,
+        leaked: Type,
+    },
 }
 
 impl Error {
@@ -521,6 +581,7 @@ pub enum UnknownValueConstructorError {
     Variable {
         name: EcoString,
         variables: Vec<EcoString>,
+        type_with_name_in_scope: bool,
     },
 
     Module {
@@ -540,10 +601,15 @@ pub fn convert_get_value_constructor_error(
     location: SrcSpan,
 ) -> Error {
     match e {
-        UnknownValueConstructorError::Variable { name, variables } => Error::UnknownVariable {
+        UnknownValueConstructorError::Variable {
+            name,
+            variables,
+            type_with_name_in_scope,
+        } => Error::UnknownVariable {
             location,
             name,
             variables,
+            type_with_name_in_scope,
         },
 
         UnknownValueConstructorError::Module {
