@@ -3,8 +3,9 @@ use crate::{
     analyse::TargetSupport,
     ast::{TypedModule, TypedStatement, UntypedExpr, UntypedModule},
     build::{Origin, Target},
+    config::PackageConfig,
     error::Error,
-    type_::{build_prelude, pretty::Printer},
+    type_::{build_prelude, expression::FunctionDefinition, pretty::Printer},
     uid::UniqueIdGenerator,
     warning::{TypeWarningEmitter, VectorWarningEmitterIO, WarningEmitter, WarningEmitterIO},
 };
@@ -21,13 +22,13 @@ mod conditional_compilation;
 mod custom_types;
 mod errors;
 mod exhaustiveness;
-mod expression;
 mod externals;
 mod functions;
 mod guards;
 mod imports;
 mod pipes;
 mod pretty;
+mod target_implementations;
 mod type_alias;
 mod use_;
 mod warnings;
@@ -80,9 +81,10 @@ macro_rules! assert_module_infer {
 macro_rules! assert_js_module_infer {
     ($src:expr, $module:expr $(,)?) => {{
         let constructors = $crate::type_::tests::infer_module_with_target(
+            "test_module",
             $src,
             vec![],
-            crate::build::Target::JavaScript,
+            $crate::build::Target::JavaScript,
         );
         let expected = $crate::type_::tests::stringify_tuple_strs($module);
         assert_eq!(($src, constructors), ($src, expected));
@@ -103,7 +105,7 @@ macro_rules! assert_js_module_error {
         let output = $crate::type_::tests::module_error_with_target(
             $src,
             vec![],
-            crate::build::Target::JavaScript,
+            $crate::build::Target::JavaScript,
         );
         insta::assert_snapshot!(insta::internals::AutoName, output, $src);
     };
@@ -164,7 +166,7 @@ macro_rules! assert_with_module_error {
 
 fn get_warnings(src: &str, deps: Vec<DependencyModule<'_>>) -> Vec<Warning> {
     let warnings = VectorWarningEmitterIO::default();
-    _ = compile_module(src, Some(Arc::new(warnings.clone())), deps).unwrap();
+    _ = compile_module("test_module", src, Some(Arc::new(warnings.clone())), deps).unwrap();
     warnings
         .take()
         .into_iter()
@@ -208,6 +210,15 @@ macro_rules! assert_warning {
         insta::assert_snapshot!(insta::internals::AutoName, output, $src);
     };
 
+    ($(($name:expr, $module_src:literal)),+, $src:expr) => {
+        let output = $crate::type_::tests::get_printed_warnings(
+            $src,
+            vec![$(("thepackage", $name, $module_src)),*]
+        );
+        assert!(!output.is_empty());
+        insta::assert_snapshot!(insta::internals::AutoName, output, $src);
+    };
+
     ($(($package:expr, $name:expr, $module_src:literal)),+, $src:expr) => {
         let output = $crate::type_::tests::get_printed_warnings(
             $src,
@@ -267,10 +278,10 @@ fn compile_statement_sequence(src: &str) -> Result<Vec1<TypedStatement>, crate::
             &TypeWarningEmitter::null(),
             TargetSupport::Enforced,
         ),
-        Implementations {
-            gleam: false,
-            uses_erlang_externals: false,
-            uses_javascript_externals: false,
+        FunctionDefinition {
+            has_body: true,
+            has_erlang_external: false,
+            has_javascript_external: false,
         },
     )
     .infer_statements(ast)
@@ -298,20 +309,28 @@ pub fn stringify_tuple_strs(module: Vec<(&str, &str)>) -> Vec<(EcoString, String
 type DependencyModule<'a> = (&'a str, &'a str, &'a str);
 
 pub fn infer_module(src: &str, dep: Vec<DependencyModule<'_>>) -> Vec<(EcoString, String)> {
-    infer_module_with_target(src, dep, Target::Erlang)
+    infer_module_with_target("test_module", src, dep, Target::Erlang)
 }
 
 pub fn infer_module_with_target(
+    module_name: &str,
     src: &str,
     dep: Vec<DependencyModule<'_>>,
     target: Target,
 ) -> Vec<(EcoString, String)> {
-    let ast =
-        compile_module_with_target(src, None, dep, target).expect("should successfully infer");
+    let ast = compile_module_with_opts(
+        module_name,
+        src,
+        None,
+        dep,
+        target,
+        TargetSupport::NotEnforced,
+    )
+    .expect("should successfully infer");
     ast.type_info
         .values
         .iter()
-        .filter(|(_, v)| v.public)
+        .filter(|(_, v)| v.publicity.is_importable())
         .map(|(k, v)| {
             let mut printer = Printer::new();
             (k.clone(), printer.pretty_print(&v.type_, 0))
@@ -321,18 +340,28 @@ pub fn infer_module_with_target(
 }
 
 pub fn compile_module(
+    module_name: &str,
     src: &str,
     warnings: Option<Arc<dyn WarningEmitterIO>>,
     dep: Vec<DependencyModule<'_>>,
 ) -> Result<TypedModule, crate::type_::Error> {
-    compile_module_with_target(src, warnings, dep, Target::Erlang)
+    compile_module_with_opts(
+        module_name,
+        src,
+        warnings,
+        dep,
+        Target::Erlang,
+        TargetSupport::NotEnforced,
+    )
 }
 
-pub fn compile_module_with_target(
+pub fn compile_module_with_opts(
+    module_name: &str,
     src: &str,
     warnings: Option<Arc<dyn WarningEmitterIO>>,
     dep: Vec<DependencyModule<'_>>,
     target: Target,
+    target_support: TargetSupport,
 ) -> Result<TypedModule, crate::type_::Error> {
     let ids = UniqueIdGenerator::new();
     let mut modules = im::HashMap::new();
@@ -355,16 +384,21 @@ pub fn compile_module_with_target(
         let parsed = crate::parse::parse_module(module_src).expect("syntax error");
         let mut ast = parsed.module;
         ast.name = name.into();
+        let line_numbers = LineNumbers::new(module_src);
+        let mut config = crate::config::PackageConfig::default();
+        config.name = package.into();
         let module = crate::analyse::infer_module::<()>(
             target,
             &ids,
             ast,
             Origin::Src,
-            &package.into(),
             &modules,
             &warnings,
             &std::collections::HashMap::from_iter(vec![]),
-            TargetSupport::NotEnforced,
+            target_support,
+            line_numbers,
+            &config,
+            "".into(),
         )
         .expect("should successfully infer");
         let _ = modules.insert(name.into(), module.type_info);
@@ -375,17 +409,22 @@ pub fn compile_module_with_target(
     }
 
     let parsed = crate::parse::parse_module(src).expect("syntax error");
-    let ast = parsed.module;
+    let mut ast = parsed.module;
+    ast.name = module_name.into();
+    let mut config = crate::config::PackageConfig::default();
+    config.name = "thepackage".into();
     crate::analyse::infer_module(
         target,
         &ids,
         ast,
         Origin::Src,
-        &"thepackage".into(),
         &modules,
         &warnings,
         &direct_dependencies,
         TargetSupport::Enforced,
+        LineNumbers::new(src),
+        &config,
+        "".into(),
     )
 }
 
@@ -398,8 +437,15 @@ pub fn module_error_with_target(
     deps: Vec<DependencyModule<'_>>,
     target: Target,
 ) -> String {
-    let error =
-        compile_module_with_target(src, None, deps, target).expect_err("should infer an error");
+    let error = compile_module_with_opts(
+        "themodule",
+        src,
+        None,
+        deps,
+        target,
+        TargetSupport::NotEnforced,
+    )
+    .expect_err("should infer an error");
     let error = Error::Type {
         src: src.into(),
         path: Utf8PathBuf::from("/src/one/two.gleam"),
@@ -566,16 +612,20 @@ fn infer_module_type_retention_test() {
     // to have one place where we create all this required state for use in each
     // place.
     let _ = modules.insert(PRELUDE_MODULE_NAME.into(), build_prelude(&ids));
+    let mut config = PackageConfig::default();
+    config.name = "thepackage".into();
     let module = crate::analyse::infer_module::<()>(
         Target::Erlang,
         &ids,
         module,
         Origin::Src,
-        &"thepackage".into(),
         &modules,
         &TypeWarningEmitter::null(),
         &direct_dependencies,
         TargetSupport::Enforced,
+        LineNumbers::new(""),
+        &config,
+        "".into(),
     )
     .expect("Should infer OK");
 
@@ -586,6 +636,7 @@ fn infer_module_type_retention_test() {
             origin: Origin::Src,
             package: "thepackage".into(),
             name: "ok".into(),
+            is_internal: false,
             // Core type constructors like String and Int are not included
             types: HashMap::new(),
             types_value_constructors: HashMap::from([
@@ -639,6 +690,8 @@ fn infer_module_type_retention_test() {
             values: HashMap::new(),
             accessors: HashMap::new(),
             unused_imports: Vec::new(),
+            line_numbers: LineNumbers::new(""),
+            src_path: "".into(),
         }
     );
 }
@@ -1973,20 +2026,46 @@ fn block_maths() {
 
 #[test]
 fn contains_todo_true() {
-    let module = compile_module("pub fn main() { 1 }", None, vec![]).unwrap();
+    let module = compile_module("test_module", "pub fn main() { 1 }", None, vec![]).unwrap();
     assert!(!module.type_info.contains_todo);
 }
 
 #[test]
 fn contains_todo_false() {
-    let module = compile_module("pub fn main() { todo }", None, vec![]).unwrap();
+    let module = compile_module("test_module", "pub fn main() { todo }", None, vec![]).unwrap();
     assert!(module.type_info.contains_todo);
 }
 
+#[test]
+fn public_type_from_internal_module_has_internal_publicity() {
+    let module = compile_module("thepackage/internal", "pub type Wibble", None, vec![]).unwrap();
+    let type_ = module.type_info.get_public_type("Wibble").unwrap();
+    assert!(type_.publicity.is_internal());
+}
+
+#[test]
+fn internal_type_from_internal_module_has_internal_publicity() {
+    let module = compile_module(
+        "thepackage/internal",
+        "@internal pub type Wibble",
+        None,
+        vec![],
+    )
+    .unwrap();
+    let type_ = module.type_info.get_public_type("Wibble").unwrap();
+    assert!(type_.publicity.is_internal());
+}
+
+#[test]
+fn private_type_from_internal_module_is_not_exposed_as_internal() {
+    let module = compile_module("thepackage/internal", "type Wibble", None, vec![]).unwrap();
+    assert!(module.type_info.get_public_type("Wibble").is_none());
+}
+
 #[test]
 fn assert_suitable_main_function_not_module_function() {
     let value = ValueConstructor {
-        public: true,
+        publicity: Publicity::Public,
         deprecation: Deprecation::NotDeprecated,
         type_: fn_(vec![], int()),
         variant: ValueConstructorVariant::ModuleConstant {
@@ -2001,6 +2080,8 @@ fn assert_suitable_main_function_not_module_function() {
                 gleam: true,
                 uses_erlang_externals: false,
                 uses_javascript_externals: false,
+                can_run_on_erlang: true,
+                can_run_on_javascript: true,
             },
         },
     };
@@ -2010,7 +2091,7 @@ fn assert_suitable_main_function_not_module_function() {
 #[test]
 fn assert_suitable_main_function_wrong_arity() {
     let value = ValueConstructor {
-        public: true,
+        publicity: Publicity::Public,
         deprecation: Deprecation::NotDeprecated,
         type_: fn_(vec![], int()),
         variant: ValueConstructorVariant::ModuleFn {
@@ -2024,6 +2105,8 @@ fn assert_suitable_main_function_wrong_arity() {
                 gleam: true,
                 uses_erlang_externals: false,
                 uses_javascript_externals: false,
+                can_run_on_erlang: true,
+                can_run_on_javascript: true,
             },
         },
     };
@@ -2033,7 +2116,7 @@ fn assert_suitable_main_function_wrong_arity() {
 #[test]
 fn assert_suitable_main_function_ok() {
     let value = ValueConstructor {
-        public: true,
+        publicity: Publicity::Public,
         deprecation: Deprecation::NotDeprecated,
         type_: fn_(vec![], int()),
         variant: ValueConstructorVariant::ModuleFn {
@@ -2047,6 +2130,8 @@ fn assert_suitable_main_function_ok() {
                 gleam: true,
                 uses_erlang_externals: false,
                 uses_javascript_externals: false,
+                can_run_on_erlang: true,
+                can_run_on_javascript: true,
             },
         },
     };
@@ -2056,30 +2141,7 @@ fn assert_suitable_main_function_ok() {
 #[test]
 fn assert_suitable_main_function_erlang_not_supported() {
     let value = ValueConstructor {
-        public: true,
-        deprecation: Deprecation::NotDeprecated,
-        type_: fn_(vec![], int()),
-        variant: ValueConstructorVariant::ModuleFn {
-            name: "name".into(),
-            field_map: None,
-            arity: 0,
-            documentation: None,
-            location: Default::default(),
-            module: "module".into(),
-            implementations: Implementations {
-                gleam: false,
-                uses_erlang_externals: false,
-                uses_javascript_externals: true,
-            },
-        },
-    };
-    assert!(assert_suitable_main_function(&value, &"module".into(), Target::Erlang).is_err(),);
-}
-
-#[test]
-fn assert_suitable_main_function_javascript_not_supported() {
-    let value = ValueConstructor {
-        public: true,
+        publicity: Publicity::Public,
         deprecation: Deprecation::NotDeprecated,
         type_: fn_(vec![], int()),
         variant: ValueConstructorVariant::ModuleFn {
@@ -2092,7 +2154,34 @@ fn assert_suitable_main_function_javascript_not_supported() {
             implementations: Implementations {
                 gleam: false,
                 uses_erlang_externals: true,
-                uses_javascript_externals: false,
+                uses_javascript_externals: true,
+                can_run_on_erlang: false,
+                can_run_on_javascript: true,
+            },
+        },
+    };
+    assert!(assert_suitable_main_function(&value, &"module".into(), Target::Erlang).is_err(),);
+}
+
+#[test]
+fn assert_suitable_main_function_javascript_not_supported() {
+    let value = ValueConstructor {
+        publicity: Publicity::Public,
+        deprecation: Deprecation::NotDeprecated,
+        type_: fn_(vec![], int()),
+        variant: ValueConstructorVariant::ModuleFn {
+            name: "name".into(),
+            field_map: None,
+            arity: 0,
+            documentation: None,
+            location: Default::default(),
+            module: "module".into(),
+            implementations: Implementations {
+                gleam: false,
+                uses_erlang_externals: true,
+                uses_javascript_externals: true,
+                can_run_on_erlang: true,
+                can_run_on_javascript: false,
             },
         },
     };
