@@ -901,7 +901,7 @@ pub fn main() {
   x
 }"#;
     let warnings = VectorWarningEmitterIO::default();
-    _ = compile_module(src, Some(Arc::new(warnings.clone())), vec![]).unwrap_err();
+    _ = compile_module("test_module", src, Some(Arc::new(warnings.clone())), vec![]).unwrap_err();
     assert!(warnings.take().is_empty());
 }
 
@@ -1150,3 +1150,320 @@ fn unused_alias_for_duplicate_module_no_warning_for_alias_test() {
         }
     );
 }
+
+#[test]
+fn result_in_case_discarded() {
+    assert_warning!(
+        "
+pub fn main(x) {
+  case x {
+    _ -> Error(Nil)
+  }
+  Nil
+}",
+        Warning::ImplicitlyDiscardedResult {
+            location: SrcSpan { start: 20, end: 52 }
+        }
+    );
+}
+
+#[test]
+fn pattern_matching_on_literal_tuple() {
+    assert_warning!(
+        "pub fn main() {
+        case #(1, 2) {
+            _ -> Nil
+        }
+      }"
+    );
+}
+
+#[test]
+fn pattern_matching_on_multiple_literal_tuples() {
+    assert_warning!(
+        "pub fn main() {
+        let wibble = 1
+        case #(1, 2), #(wibble, wibble) {
+            _, _ -> Nil
+        }
+      }"
+    );
+}
+
+#[test]
+fn pattern_matching_on_tuples_doesnt_raise_a_warning() {
+    assert_no_warnings!(
+        "pub fn main() {
+        let wibble = #(1, 2)
+        // This doesn't raise a warning since `wibble` is not a literal tuple.
+        case wibble {
+            _ -> Nil
+        }
+      }"
+    );
+}
+
+#[test]
+fn opaque_external_type_raises_a_warning() {
+    assert_warning!("pub opaque type External");
+}
+
+#[test]
+fn unused_binary_operation_raises_a_warning() {
+    assert_warning!(
+        r#"
+pub fn main() {
+  let string = "a" <> "b" "c" <> "d"
+  string
+}
+"#
+    );
+}
+
+#[test]
+fn unused_record_access_raises_a_warning() {
+    assert_warning!(
+        r#"
+pub type Thing {
+  Thing(value: Int)
+}
+
+pub fn main() {
+  let thing = Thing(1)
+  thing.value
+  1
+}
+"#
+    );
+}
+
+#[test]
+fn unused_record_constructor_raises_a_warning() {
+    assert_warning!(
+        r#"
+pub type Thing {
+  Thing(value: Int)
+}
+
+pub fn main() {
+  Thing(1)
+  1
+}
+"#
+    );
+}
+
+#[test]
+fn unused_record_update_raises_a_warning() {
+    assert_warning!(
+        r#"
+pub type Thing {
+  Thing(value: Int, other: Int)
+}
+
+pub fn main() {
+  let thing = Thing(1, 2)
+  Thing(..thing, value: 1)
+  1
+}
+"#
+    );
+}
+
+#[test]
+fn unused_variable_raises_a_warning() {
+    assert_warning!(
+        r#"
+pub fn main() {
+  let number = 1
+  number
+  1
+}
+"#
+    );
+}
+
+#[test]
+fn unused_function_literal_raises_a_warning() {
+    assert_warning!(
+        r#"
+pub fn main() {
+  fn(n) { n + 1 }
+  1
+}
+"#
+    );
+}
+
+#[test]
+fn unused_tuple_index_raises_a_warning() {
+    assert_warning!(
+        r#"
+pub fn main() {
+  #(1, 2).0
+  1
+}
+"#
+    );
+}
+
+#[test]
+fn unused_bool_negation_raises_a_warning() {
+    assert_warning!(
+        r#"
+pub fn main() {
+  !True
+  1
+}
+"#
+    );
+}
+
+#[test]
+fn unused_int_negation_raises_a_warning() {
+    assert_warning!(
+        r#"
+pub fn main() {
+  -1
+  1
+}
+"#
+    );
+}
+
+#[test]
+fn unused_pipeline_ending_with_variant_raises_a_warning() {
+    assert_warning!(
+        r#"
+pub type Wibble(a) { Wibble(a) }
+pub fn wibble(a) { a }
+
+pub fn main() {
+  1 |> wibble |> Wibble
+  1
+}
+"#
+    );
+}
+
+#[test]
+fn unused_pipeline_not_ending_with_variant_raises_no_warnings() {
+    assert_no_warnings!(
+        r#"
+pub type Wibble(a) { Wibble(a) }
+pub fn wibble(a) { a }
+
+pub fn main() {
+  1 |> wibble |> wibble
+  1
+}
+"#
+    );
+}
+
+/*
+
+TODO: These tests are commented out until we figure out a better way to deal
+      with reexports of internal types and reintroduce the warning.
+      As things stand it would break both Lustre and Mist.
+      You can see the thread starting around here for more context:
+      https://discord.com/channels/768594524158427167/768594524158427170/1227250677734969386
+
+#[test]
+fn internal_type_in_public_function_return() {
+    assert_warning!(
+        "
+@internal
+pub type Wibble {
+  Wibble
+}
+
+pub fn wibble() -> Wibble { Wibble }
+"
+    );
+}
+
+#[test]
+fn type_from_internal_module_in_public_function_return() {
+    assert_warning!(
+        ("thepackage/internal", "pub type Wibble { Wibble }"),
+        "
+import thepackage/internal.{type Wibble, Wibble}
+
+pub fn wibble() -> Wibble {
+  Wibble
+}"
+    );
+}
+
+#[test]
+fn internal_type_in_public_function_argument() {
+    assert_warning!(
+        "
+@internal
+pub type Wibble {
+  Wibble
+}
+
+pub fn wibble(_wibble: Wibble) -> Int { 1 }
+"
+    );
+}
+
+#[test]
+fn type_from_internal_module_in_public_function_argument() {
+    assert_warning!(
+        ("thepackage/internal", "pub type Wibble { Wibble }"),
+        "
+import thepackage/internal.{type Wibble}
+
+pub fn wibble(_wibble: Wibble) -> Int {
+  1
+}
+"
+    );
+}
+
+#[test]
+fn internal_type_in_public_constructor() {
+    assert_warning!(
+        "
+@internal
+pub type Wibble {
+  Wibble
+}
+
+pub type Wobble {
+    Wobble(Wibble)
+}
+"
+    );
+}
+
+#[test]
+fn type_from_internal_module_in_public_constructor() {
+    assert_warning!(
+        ("thepackage/internal", "pub type Wibble { Wibble }"),
+        "
+import thepackage/internal.{type Wibble}
+
+pub type Wobble {
+  Wobble(Wibble)
+}"
+    );
+}
+
+#[test]
+fn type_from_internal_module_dependency_in_public_constructor() {
+    assert_warning!(
+        ("dep", "dep/internal", "pub type Wibble { Wibble }"),
+        "
+import dep/internal.{type Wibble}
+
+pub type Wobble {
+  Wobble(Wibble)
+}"
+    );
+}
+
+*/
