@@ -2,7 +2,7 @@ use crate::{
     ast::TodoKind,
     diagnostic::{self, Diagnostic, Location},
     error::wrap,
-    type_,
+    type_::{self, pretty::Printer},
 };
 use camino::Utf8PathBuf;
 use debug_ignore::DebugIgnore;
@@ -154,8 +154,9 @@ impl Warning {
         match self {
             Warning::InvalidSource { path } => Diagnostic {
                 title: "Invalid module name".into(),
-                text: "Module names must begin with a lowercase letter and contain\
- only lowercase alphanumeric characters or underscores."
+                text: "\
+Module names must begin with a lowercase letter and contain
+only lowercase alphanumeric characters or underscores."
                     .into(),
                 level: diagnostic::Level::Warning,
                 location: None,
@@ -598,6 +599,91 @@ the same values.\n"
                         }),
                     }
                 }
+
+                type_::Warning::CaseMatchOnLiteralTuple { location } => Diagnostic {
+                    title: "Redundant tuple".into(),
+                    text: "Case expressions can take multiple subjects directly.".into(),
+                    hint: Some(
+                        "You can pass the contents of the tuple directly, separated by commas."
+                            .into(),
+                    ),
+                    level: diagnostic::Level::Warning,
+                    location: Some(Location {
+                        src: src.clone(),
+                        path: path.to_path_buf(),
+                        label: diagnostic::Label {
+                            text: Some("You can remove this tuple wrapper".into()),
+                            span: *location,
+                        },
+                        extra_labels: Vec::new(),
+                    }),
+                },
+
+                type_::Warning::OpaqueExternalType { location } => Diagnostic {
+                    title: "Opaque external type".into(),
+                    text: "This type has no constructors so making it opaque is redundant.".into(),
+                    hint: Some("Remove the `opaque` qualifier from the type definition.".into()),
+                    level: diagnostic::Level::Warning,
+                    location: Some(Location {
+                        src: src.clone(),
+                        path: path.to_path_buf(),
+                        label: diagnostic::Label {
+                            text: None,
+                            span: *location,
+                        },
+                        extra_labels: Vec::new(),
+                    }),
+                },
+
+                type_::Warning::UnusedValue { location } => Diagnostic {
+                    title: "Unused value".into(),
+                    text: "".into(),
+                    hint: None,
+                    level: diagnostic::Level::Warning,
+                    location: Some(Location {
+                        path: path.to_path_buf(),
+                        src: src.clone(),
+                        label: diagnostic::Label {
+                            text: Some("This value is never used".into()),
+                            span: *location,
+                        },
+                        extra_labels: Vec::new(),
+                    }),
+                },
+
+                type_::Warning::InternalTypeLeak { location, leaked } => {
+                    let mut printer = Printer::new();
+
+                    // TODO: be more precise.
+                    // - is being returned by this public function
+                    // - is taken as an argument by this public function
+                    // - is taken as an argument by this public enum constructor
+                    // etc
+                    let text = format!(
+                        "The following type is internal, but is being used by this public export.
+
+{}
+
+Internal types should not be used in a public facing function since they are
+hidden from the package's documentation.",
+                        printer.pretty_print(leaked, 4),
+                    );
+                    Diagnostic {
+                        title: "Internal type used in public interface".into(),
+                        text,
+                        hint: None,
+                        level: diagnostic::Level::Warning,
+                        location: Some(Location {
+                            label: diagnostic::Label {
+                                text: None,
+                                span: *location,
+                            },
+                            path: path.clone(),
+                            src: src.clone(),
+                            extra_labels: vec![],
+                        }),
+                    }
+                }
             },
         }
     }
